from ptsl.ops import Operation


class TogglePlayState(Operation):
    pass


class ToggleRecordEnable(Operation):
    pass


class PlayHalfSpeed(Operation):
    pass


class RecordHalfSpeed(Operation):
    pass



from ptsl import ops
from ptsl.PTSL_pb2 import SessionData, AudioData, \
    Session, \
    LinkToSourceAudio, CopyFromSourceAudio, ConsolidateFromSourceAudio, \
    ForceToTargetSessionFormat, \
    LinkToSourceVideo, CopyFromSourceVideo, \
    MT_None, MT_MatchTracks, MT_ImportAsNewTrack, \
    ImportReplaceExistingPlaylists, ImportOverlayNewOnExistingPlaylists, \
    DoNotImport, \
    TrackDataToImport, \
    MaintainAbsoluteTimeCodeValues, MaintainRelativeTimeCodeValues, \
    MapStartTimeCodeTo
# AddAudio, CopyAudio, ConvertAudio, Default, \

# AudioOperations, MediaDestination, MediaLocation


class ImportSessionDataBuilder:

    def __init__(self, engine, session_path: str):
        self._engine = engine
        self._session_path = session_path
        self._import_type = Session  # the only type supported at present
        self._audio_media_option = LinkToSourceAudio
        self._video_media_option = LinkToSourceVideo
        self._handles_ms = 1000

        self._match_track_option = MT_None
        self._playlist_option = DoNotImport
        self._track_data_preset = None
        self._import_clip_gain = False
        self._import_clips_and_media = False
        self._import_volume_automation = False

        self._timecode_mapping_option = MaintainAbsoluteTimeCodeValues
        self._timecode_mapping_start_time = ""
        self._adjust_session_start = False

    def link_to_source_audio(self):
        self._audio_media_option = LinkToSourceAudio

    def copy_from_source_audio(self):
        self._audio_media_option = CopyFromSourceAudio

    def consolidate_from_source_audio(self, handles_ms: int = 1000):
        self._audio_media_option = ConsolidateFromSourceAudio
        self._handles_ms = handles_ms

    def force_audio_to_session_format(self):
        self._audio_media_option = ForceToTargetSessionFormat

    def link_to_source_video(self):
        self._video_media_option = LinkToSourceVideo

    def copy_from_source_video(self):
        self._video_media_option = CopyFromSourceVideo

    def match_tracks(self):
        self._match_track_option = MT_MatchTracks

    def import_as_new_tracks(self):
        self._match_track_option = MT_ImportAsNewTrack

    def replace_playlists(self):
        self._playlist_option = ImportReplaceExistingPlaylists

    def overlay_playlists(self):
        self._playlist_option = ImportOverlayNewOnExistingPlaylists

    def use_track_data_preset(self, preset_path: str):
        self._track_data_preset = preset_path

    def import_clip_gain(self):
        self._import_clip_gain = True

    def import_clips_and_media(self):
        self._import_clips_and_media = True

    def import_volume_automation(self):
        self._import_volume_automation = True

    def maintain_absolute_timecode(self):
        self._timecode_mapping_option = MaintainAbsoluteTimeCodeValues

    def maintain_relative_timecode(self):
        self._timecode_mapping_option = MaintainRelativeTimeCodeValues

    def map_start_timecode(self, to_time: str):
        self._timecode_mapping_option = MapStartTimeCodeTo
        self._timecode_mapping_start_time = to_time

    def adjust_start_timecode_to_match_source(self):
        self._adjust_session_start = True

    def import_data(self):
        import_type = self._import_type

        track_data_import = TrackDataToImport(
            track_data_preset_path=self._track_data_preset,
            clip_gain=self._import_clip_gain,
            clips_and_media=self._import_clips_and_media,
            volume_automation=self._import_volume_automation
        )

        adj = self._adjust_session_start
        session_data = SessionData(
            audio_options=self._audio_media_option,
            audio_handle_size=self._handles_ms,
            video_options=self._video_media_option,
            match_options=self._match_track_option,
            playlist_options=self._playlist_option,
            track_data_to_import=track_data_import,
            timecode_mapping_units=self._timecode_mapping_option,
            timecode_mapping_start_time=self._timecode_mapping_start_time,
            adjust_session_start_time_to_match_source=adj,
        )

        audio_data = AudioData(
            file_list=None,
            audio_operations=None,
            destination_path=None,
            destination=None,
            location=None
        )

        op = ops.Import(
            session_path=self._session_path,
            import_type=import_type,
            session_data=session_data,
            audio_data=audio_data
        )

        self._engine.client.run(op)



import json

from ptsl.ops import Operation


class GetSessionInterleavedState(Operation):

    # Reported https://duc.avid.com/showthread.php?t=423087
    #
    # Pro Tools is returning the "possible_settings" list as
    # a list of strings and not a list of bools
    def json_cleanup(self, in_json: str) -> str:
        decoder = json.decoder.JSONDecoder()
        parsed = decoder.decode(in_json)
        parsed["possible_settings"] = [True, False]
        encoder = json.encoder.JSONEncoder()
        return encoder.encode(parsed)



"""
ptsl Exceptions
"""

from typing import Optional

from ptsl.PTSL_pb2 import CommandError as CommandErrorResponse
from ptsl.PTSL_pb2 import CommandErrorType


class CommandError(RuntimeError):
    """
    A :class:`Exception` for :class:`CommandError` results
    """
    error_response: CommandErrorResponse

    def __init__(self, error_response: CommandErrorResponse) -> None:
        self.error_response = error_response
        super().__init__()

    def __str__(self) -> str:
        return super().__str__() + \
            f"ErrType {self.error_type}: {self.error_name} ({self.message})"

    @property
    def is_warning(self) -> bool:
        """
        `True` if command error message is marked as a warning.
        """
        return self.error_response.is_warning

    @property
    def error_type(self) -> CommandErrorType:
        """
        :class:`ptsl.PTSL_pb2.CommandErrorType` enumeration value.
        """
        return self.error_response.command_error_type

    @property
    def error_name(self) -> Optional[str]:
        """
        Error name, if available.
        """
        if self.error_type in CommandErrorType.values():
            return CommandErrorType.Name(self.error_type)

        return None

    @property
    def message(self) -> str:
        """
        Error message as returned by the client.
        """
        return self.error_response.command_error_message



from typing import Optional

from ptsl import PTSL_pb2 as pt

from ptsl.ops import Operation


class GetSessionSampleRate(Operation):

    def sample_rate(self) -> Optional[float]:
        if self.response.sample_rate == pt.SR_192000:
            return 192000.0
        elif self.response.sample_rate == pt.SR_176400:
            return 176400.0
        elif self.response.sample_rate == pt.SR_96000:
            return 96000.0
        elif self.response.sample_rate == pt.SR_48000:
            return 48000.0
        elif self.response.sample_rate == pt.SR_44100:
            return 44100.0
        else:
            return None



"""
Operations classes.
"""

from typing import Any

from .operation import Operation

# Implemented by Pro Tools 2022.12

from .get_ptsl_version import GetPTSLVersion
from .get_session_sample_rate import GetSessionSampleRate
from .get_session_name import GetSessionName
from .get_session_path import GetSessionPath
from .get_session_audio_format import GetSessionAudioFormat
from .get_session_bit_depth import GetSessionBitDepth
from .get_session_start_time import GetSessionStartTime
from .get_session_time_code_rate import GetSessionTimeCodeRate
from .get_session_feet_frames_rate import GetSessionFeetFramesRate
from .get_session_interleaved_state import GetSessionInterleavedState
from .get_session_audio_rate_pull_settings import \
    GetSessionAudioRatePullSettings
from .get_session_video_rate_pull_settings import \
    GetSessionVideoRatePullSettings
from .get_session_length import GetSessionLength
from .get_playback_mode import GetPlaybackMode
from .get_transport_state import GetTransportState
from .get_transport_armed import GetTransportArmed
from .get_record_mode import GetRecordMode
from .get_track_list import GetTrackList
from .create_session import CreateSession
from .open_session import OpenSession
from .export_session_info_as_text import ExportSessionInfoAsText
from .select_all_clips_on_track import SelectAllClipsOnTrack
from .extend_selection_to_target_tracks import ExtendSelectionToTargetTracks
from .create_fades_based_on_preset import CreateFadesBasedOnPreset
from .cut import Cut
from .copy import Copy
from .paste import Paste
from .clear import Clear
from .save_session import SaveSession
from .close_session import CloseSession
from .trim_to_selection import TrimToSelection
from .consolidate_clip import ConsolidateClip
from .import_command import Import
from .clear_special import ClearSpecial
from .copy_special import CopySpecial
from .cut_special import CutSpecial
from .export_clips_as_files import ExportClipsAsFiles
from .export_selected_tracks_as_aaf_omf import ExportSelectedTracksAsAAFOMF
from .export_mix import ExportMix
from .get_dyamic_properties import GetDynamicProperties
from .get_file_location import GetFileLocation
from .get_task_status import GetTaskStatus
from .host_ready_check import HostReadyCheck
from .paste_special import PasteSpecial
from .refresh_all_modified_audio_files import RefreshAllModifiedAudioFiles
from .refresh_target_audio_files import RefreshTargetAudioFiles
from .rename_target_track import RenameTargetTrack
from .save_session_as import SaveSessionAs
from .set_playback_mode import SetPlaybackMode
from .set_record_mode import SetRecordMode
from .spot import Spot
from .authorize_connection import AuthorizeConnection
from .set_session_audio_format import SetSessionAudioFormat
from .set_session_audio_rate_pull_settings import \
    SetSessionAudioRatePullSettings
from .set_session_bit_depth import SetSessionBitDepth
from .set_session_feet_frames_rate import SetSessionFeetFramesRate
from .set_session_interleaved_state import SetSessionInterleavedState
from .set_session_length import SetSessionLength
from .set_session_start_time import SetSessionStartTime
from .set_session_time_code_rate import SetSessionTimeCodeRate
from .set_session_video_rate_pull_settings import \
    SetSessionVideoRatePullSettings

# Pro Tools 2023.3

from .register_connection import RegisterConnection
from .rename_selected_clip import RenameSelectedClip
from .rename_target_clip import RenameTargetClip
from .toggle_play_state import \
    TogglePlayState, ToggleRecordEnable, \
    PlayHalfSpeed, RecordHalfSpeed

from .memory_locations import EditMemoryLocation, GetMemoryLocations



from ptsl.PTSL_pb2 import SAF_AIFF, SAF_WAVE,\
    SR_48000, Bit16, Bit24, Bit32Float, \
    IO_Last, IO_StereoMix, IO_51SMPTEMix

import ptsl
from ptsl import ops, util


class CreateSessionBuilder:
    def __init__(self, engine: 'ptsl.Engine', name: str, path: str):
        self._engine = engine
        self._session_name = name
        self._path = path
        self._audio_format = SAF_WAVE
        self._sample_rate = SR_48000
        self._bit_depth = Bit24
        self._io_settings = IO_Last
        self._is_interleaved = False

    def audio_format(self, value: str):
        """
        :param value: Audio format for the new session. Acceptable
            values are "wave" or "aiff".
        """
        if value == 'wave':
            self._audio_format = SAF_WAVE
        elif value == 'aiff':
            self._audio_format = SAF_AIFF
        else:
            assert False, f"Invalid audio_format value {value}"

    def wave_format(self):
        self.audio_format("wave")

    def aiff_format(self):
        self.audio_format("aiff")

    def sample_rate(self, value: int):
        self._sample_rate = util.sample_rate_enum(value)

    def bit_depth(self, value: int):
        """
        :param value: Bit depth for the new session. Acceptable
            values are `16`, `24` or `32`.
        """
        if value == 16:
            self._bit_depth = Bit16
        elif value == 24:
            self._bit_depth = Bit24
        elif value == 32:
            self._bit_depth = Bit32Float
        else:
            assert False, f"Invalid bit_depth value {value}"

    def stereo_io_settings(self):
        self._io_settings = IO_StereoMix

    def smpte51_io_settings(self):
        self._io_settings = IO_51SMPTEMix

    def interlaved(self, value: bool):
        self._is_interleaved = value

    def create(self) -> None:
        op = ops.CreateSession(
            session_name=self._session_name,
            file_type=self._audio_format,
            sample_rate=self._sample_rate,
            input_output_settings=self._io_settings,
            is_interleaved=self._is_interleaved,
            session_location=self._path,
            bit_depth=self._bit_depth,
        )

        self._engine.client.run(op)


class CreateSessionFromTemplateBuilder(CreateSessionBuilder):

    def __init__(self, engine: 'ptsl.Engine',
                 template_name: str,
                 template_group: str,
                 name: str,
                 path: str):
        self._template_name = template_name
        self._template_group = template_group
        super().__init__(engine, name, path)

    def create(self):
        op = ops.CreateSession(
            session_name=self._session_name,
            create_from_template=True,
            template_group=self._template_group,
            template_name=self._template_name,
            file_type=self._audio_format,
            sample_rate=self._sample_rate,
            input_output_settings=self._io_settings,
            is_interleaved=self._is_interleaved,
            session_location=self._path,
            bit_depth=self._bit_depth
        )

        self._engine.client.run(op)


class CreateSessionFromAAFBuilder(CreateSessionBuilder):

    def __init__(self, engine: 'ptsl.Engine',
                 aaf_path: str,
                 name: str,
                 path: str):
        self._aaf_path = aaf_path
        super().__init__(engine, name, path)

    def create(self):
        op = ops.CreateSession(
            session_name=self._session_name,
            file_type=self._audio_format,
            sample_rate=self._sample_rate,
            input_output_settings=self._io_settings,
            is_interleaved=self._is_interleaved,
            session_location=self._path,
            bit_depth=self._bit_depth,
            create_from_aaf=True,
            path_to_aaf=self._aaf_path
        )

        self._engine.client.run(op)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: PTSL.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\nPTSL.proto\x12\x04ptsl\"\x0e\n\x0c\x45mptyMessage\"u\n\x0c\x43ommandError\x12\x32\n\x12\x63ommand_error_type\x18\x01 \x01(\x0e\x32\x16.ptsl.CommandErrorType\x12\x1d\n\x15\x63ommand_error_message\x18\x02 \x01(\t\x12\x12\n\nis_warning\x18\x03 \x01(\x08\"g\n\rRequestHeader\x12\x0f\n\x07task_id\x18\x01 \x01(\t\x12 \n\x07\x63ommand\x18\x02 \x01(\x0e\x32\x0f.ptsl.CommandId\x12\x0f\n\x07version\x18\x03 \x01(\x05\x12\x12\n\nsession_id\x18\x04 \x01(\t\"I\n\x07Request\x12#\n\x06header\x18\x01 \x01(\x0b\x32\x13.ptsl.RequestHeader\x12\x19\n\x11request_body_json\x18\x02 \x01(\t\"w\n\x0eResponseHeader\x12\x0f\n\x07task_id\x18\x01 \x01(\t\x12 \n\x07\x63ommand\x18\x02 \x01(\x0e\x32\x0f.ptsl.CommandId\x12 \n\x06status\x18\x03 \x01(\x0e\x32\x10.ptsl.TaskStatus\x12\x10\n\x08progress\x18\x04 \x01(\x05\"i\n\x08Response\x12$\n\x06header\x18\x01 \x01(\x0b\x32\x14.ptsl.ResponseHeader\x12\x1a\n\x12response_body_json\x18\x02 \x01(\t\x12\x1b\n\x13response_error_json\x18\x03 \x01(\t\":\n\nPagination\x12\r\n\x05total\x18\x01 \x01(\x05\x12\r\n\x05limit\x18\x02 \x01(\x05\x12\x0e\n\x06offset\x18\x03 \x01(\x05\"\xb2\x03\n\x0fTrackAttributes\x12.\n\x0bis_inactive\x18\x01 \x01(\x0e\x32\x19.ptsl.TrackAttributeState\x12,\n\tis_hidden\x18\x02 \x01(\x0e\x32\x19.ptsl.TrackAttributeState\x12.\n\x0bis_selected\x18\x03 \x01(\x0e\x32\x19.ptsl.TrackAttributeState\x12\x16\n\x0e\x63ontains_clips\x18\x04 \x01(\x08\x12\x1b\n\x13\x63ontains_automation\x18\x05 \x01(\x08\x12\x11\n\tis_soloed\x18\x06 \x01(\x08\x12\x19\n\x11is_record_enabled\x18\x07 \x01(\x08\x12\x39\n\x16is_input_monitoring_on\x18\x08 \x01(\x0e\x32\x19.ptsl.TrackAttributeState\x12\x17\n\x0fis_smart_dsp_on\x18\t \x01(\x08\x12\x11\n\tis_locked\x18\n \x01(\x08\x12\x10\n\x08is_muted\x18\x0b \x01(\x08\x12\x11\n\tis_frozen\x18\x0c \x01(\x08\x12\x0f\n\x07is_open\x18\r \x01(\x08\x12\x11\n\tis_online\x18\x0e \x01(\x08\"\xa6\x01\n\x05Track\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x1d\n\x04type\x18\x02 \x01(\x0e\x32\x0f.ptsl.TrackType\x12\n\n\x02id\x18\x03 \x01(\t\x12\r\n\x05index\x18\x04 \x01(\x05\x12\r\n\x05\x63olor\x18\x05 \x01(\t\x12/\n\x10track_attributes\x18\x06 \x01(\x0b\x32\x15.ptsl.TrackAttributes\x12\x15\n\rid_compressed\x18\x07 \x01(\t\"+\n\x18GetTaskStatusRequestBody\x12\x0f\n\x07task_id\x18\x01 \x01(\t\"`\n\x19GetTaskStatusResponseBody\x12\x0f\n\x07task_id\x18\x01 \x01(\t\x12 \n\x06status\x18\x02 \x01(\x0e\x32\x10.ptsl.TaskStatus\x12\x10\n\x08progress\x18\x03 \x01(\x05\"\x95\x03\n\x18\x43reateSessionRequestBody\x12\x14\n\x0csession_name\x18\x01 \x01(\t\x12\x1c\n\x14\x63reate_from_template\x18\x02 \x01(\x08\x12\x16\n\x0etemplate_group\x18\x03 \x01(\t\x12\x15\n\rtemplate_name\x18\x04 \x01(\t\x12!\n\tfile_type\x18\x05 \x01(\x0e\x32\x0e.ptsl.FileType\x12%\n\x0bsample_rate\x18\x06 \x01(\x0e\x32\x10.ptsl.SampleRate\x12/\n\x15input_output_settings\x18\x07 \x01(\x0e\x32\x10.ptsl.IOSettings\x12\x16\n\x0eis_interleaved\x18\x08 \x01(\x08\x12\x18\n\x10session_location\x18\t \x01(\t\x12\x18\n\x10is_cloud_project\x18\n \x01(\x08\x12\x17\n\x0f\x63reate_from_aaf\x18\x0b \x01(\x08\x12\x13\n\x0bpath_to_aaf\x18\x0c \x01(\t\x12!\n\tbit_depth\x18\r \x01(\x0e\x32\x0e.ptsl.BitDepth\".\n\x16OpenSessionRequestBody\x12\x14\n\x0csession_path\x18\x01 \x01(\t\"z\n\x11TrackDataToImport\x12\x1e\n\x16track_data_preset_path\x18\x01 \x01(\t\x12\x11\n\tclip_gain\x18\x02 \x01(\x08\x12\x17\n\x0f\x63lips_and_media\x18\x03 \x01(\x08\x12\x19\n\x11volume_automation\x18\x04 \x01(\x08\"\xc6\x01\n\x11SessionDataImport\x12\x17\n\x0ftempo_meter_map\x18\x01 \x01(\x08\x12\x1f\n\x17key_signature_choed_map\x18\x02 \x01(\x08\x12 \n\x18markers_memory_locations\x18\x03 \x01(\x08\x12\x1d\n\x15window_configurations\x18\x04 \x01(\x08\x12\x18\n\x10mic_pre_settings\x18\x05 \x01(\x08\x12\x1c\n\x14heat_master_settings\x18\x06 \x01(\x08\"\xba\x03\n\x0bSessionData\x12.\n\raudio_options\x18\x01 \x01(\x0e\x32\x17.ptsl.AudioMediaOptions\x12\x19\n\x11\x61udio_handle_size\x18\x02 \x01(\x05\x12.\n\rvideo_options\x18\x03 \x01(\x0e\x32\x17.ptsl.VideoMediaOptions\x12.\n\rmatch_options\x18\x04 \x01(\x0e\x32\x17.ptsl.MatchTrackOptions\x12\x33\n\x10playlist_options\x18\x05 \x01(\x0e\x32\x19.ptsl.MainPlaylistOptions\x12\x35\n\x14track_data_to_import\x18\x06 \x01(\x0b\x32\x17.ptsl.TrackDataToImport\x12<\n\x16timecode_mapping_units\x18\x07 \x01(\x0e\x32\x1c.ptsl.TimeCodeMappingOptions\x12#\n\x1btimecode_mapping_start_time\x18\x08 \x01(\t\x12\x31\n)adjust_session_start_time_to_match_source\x18\t \x01(\x08\"\x8d\x01\n\x10SpotLocationData\x12-\n\rlocation_type\x18\x01 \x01(\x0e\x32\x16.ptsl.SpotLocationType\x12\x16\n\x0elocation_value\x18\x02 \x01(\t\x12\x32\n\x10location_options\x18\x03 \x01(\x0e\x32\x18.ptsl.TrackOffsetOptions\"\xec\x01\n\tAudioData\x12\x11\n\tfile_list\x18\x01 \x03(\t\x12/\n\x10\x61udio_operations\x18\x02 \x01(\x0e\x32\x15.ptsl.AudioOperations\x12\x18\n\x10\x64\x65stination_path\x18\x03 \x01(\t\x12+\n\x0b\x64\x65stination\x18\x04 \x01(\x0e\x32\x16.ptsl.MediaDestination\x12%\n\x08location\x18\x05 \x01(\x0e\x32\x13.ptsl.MediaLocation\x12-\n\rlocation_data\x18\x06 \x01(\x0b\x32\x16.ptsl.SpotLocationData\"\x0b\n\tVideoData\"\n\n\x08MidiData\"\x10\n\x0e\x43lipGroupsData\"\x9e\x01\n\x11ImportRequestBody\x12\x14\n\x0csession_path\x18\x01 \x01(\t\x12%\n\x0bimport_type\x18\x02 \x01(\x0e\x32\x10.ptsl.ImportType\x12\'\n\x0csession_data\x18\x03 \x01(\x0b\x32\x11.ptsl.SessionData\x12#\n\naudio_data\x18\x04 \x01(\x0b\x32\x0f.ptsl.AudioData\"r\n\x12ImportResponseBody\x12\x11\n\tfile_list\x18\x01 \x03(\t\x12/\n\x10\x61udio_operations\x18\x02 \x01(\x0e\x32\x15.ptsl.AudioOperations\x12\x18\n\x10\x64\x65stination_path\x18\x03 \x01(\t\"W\n\x19TrackListInvertibleFilter\x12%\n\x06\x66ilter\x18\x01 \x01(\x0e\x32\x15.ptsl.TrackListFilter\x12\x13\n\x0bis_inverted\x18\x02 \x01(\x08\"\x8a\x01\n\x17GetTrackListRequestBody\x12\x12\n\npage_limit\x18\x01 \x01(\x05\x12:\n\x11track_filter_list\x18\x02 \x03(\x0b\x32\x1f.ptsl.TrackListInvertibleFilter\x12\x1f\n\x17is_filter_list_additive\x18\x03 \x01(\x08\"\\\n\x18GetTrackListResponseBody\x12\x1f\n\x05stats\x18\x01 \x01(\x0b\x32\x10.ptsl.Pagination\x12\x1f\n\ntrack_list\x18\x02 \x03(\x0b\x32\x0b.ptsl.Track\"6\n SelectAllClipsOnTrackRequestBody\x12\x12\n\ntrack_name\x18\x01 \x01(\t\"7\n!SelectAllClipsOnTrackResponseBody\x12\x12\n\ntrack_name\x18\x01 \x01(\t\"G\n(ExtendSelectionToTargetTracksRequestBody\x12\x1b\n\x13tracks_to_extend_to\x18\x01 \x03(\t\"H\n)ExtendSelectionToTargetTracksResponseBody\x12\x1b\n\x13tracks_to_extend_to\x18\x01 \x03(\t\"[\n#CreateFadesBasedOnPresetRequestBody\x12\x18\n\x10\x66\x61\x64\x65_preset_name\x18\x01 \x01(\t\x12\x1a\n\x12\x61uto_adjust_bounds\x18\x02 \x01(\x08\"@\n$CreateFadesBasedOnPresetResponseBody\x12\x18\n\x10\x66\x61\x64\x65_preset_name\x18\x01 \x01(\t\"X\n\x1cRenameTargetTrackRequestBody\x12\x10\n\x08track_id\x18\x01 \x01(\t\x12\x10\n\x08new_name\x18\x02 \x01(\t\x12\x14\n\x0c\x63urrent_name\x18\x03 \x01(\t\"\xfe\x01\n\x1d\x45xportClipsAsFilesRequestBody\x12\x11\n\tfile_path\x18\x01 \x01(\t\x12\"\n\x06\x66ormat\x18\x02 \x01(\x0e\x32\x12.ptsl.ExportFormat\x12\'\n\tfile_type\x18\x03 \x01(\x0e\x32\x14.ptsl.ExportFileType\x12!\n\tbit_depth\x18\x04 \x01(\x0e\x32\x0e.ptsl.BitDepth\x12\x36\n\x0f\x64uplicate_names\x18\x05 \x01(\x0e\x32\x1d.ptsl.ResolveDuplicateNamesBy\x12\"\n\x1a\x65nforce_avid_compatibility\x18\x06 \x01(\x08\"\xad\x03\n\'ExportSelectedTracksAsAAFOMFRequestBody\x12,\n\tfile_type\x18\x02 \x01(\x0e\x32\x19.ptsl.ExportAsAAFFileType\x12(\n\tbit_depth\x18\x03 \x01(\x0e\x32\x15.ptsl.AAFFileBitDepth\x12%\n\x0b\x63opy_option\x18\x04 \x01(\x0e\x32\x10.ptsl.CopyOption\x12,\n$enforce_media_composer_compatibility\x18\x05 \x01(\x08\x12*\n\"quantize_edits_to_frame_boundaries\x18\x06 \x01(\x08\x12%\n\x1d\x65xport_stereo_as_multichannel\x18\x07 \x01(\x08\x12\x1b\n\x13\x63ontainer_file_name\x18\t \x01(\t\x12\x1f\n\x17\x63ontainer_file_location\x18\n \x01(\t\x12\x1b\n\x13\x61sset_file_location\x18\x0b \x01(\t\x12\x10\n\x08\x63omments\x18\x0c \x01(\t\x12\x15\n\rsequence_name\x18\r \x01(\t\"7\n\"RefreshTargetAudioFilesRequestBody\x12\x11\n\tfile_list\x18\x01 \x03(\t\"i\n#RefreshTargetAudioFilesResponseBody\x12\x15\n\rsuccess_count\x18\x01 \x01(\x05\x12\x15\n\rfailure_count\x18\x02 \x01(\x05\x12\x14\n\x0c\x66\x61ilure_list\x18\x03 \x03(\t\"%\n\x10\x46ileLocationInfo\x12\x11\n\tis_online\x18\x01 \x01(\x08\"B\n\x0c\x46ileLocation\x12\x0c\n\x04path\x18\x01 \x01(\t\x12$\n\x04info\x18\x02 \x01(\x0b\x32\x16.ptsl.FileLocationInfo\"d\n\x1aGetFileLocationRequestBody\x12\x12\n\npage_limit\x18\x01 \x01(\x05\x12\x32\n\x0c\x66ile_filters\x18\x02 \x03(\x0e\x32\x1c.ptsl.FileLocationTypeFilter\"j\n\x1bGetFileLocationResponseBody\x12\x1f\n\x05stats\x18\x01 \x01(\x0b\x32\x10.ptsl.Pagination\x12*\n\x0e\x66ile_locations\x18\x02 \x03(\x0b\x32\x12.ptsl.FileLocation\"0\n\x17\x43loseSessionRequestBody\x12\x15\n\rsave_on_close\x18\x01 \x01(\x08\"J\n\x18SaveSessionAsRequestBody\x12\x14\n\x0csession_name\x18\x01 \x01(\t\x12\x18\n\x10session_location\x18\x02 \x01(\t\"T\n\x15\x43utSpecialRequestBody\x12;\n\x16\x61utomation_data_option\x18\x01 \x01(\x0e\x32\x1b.ptsl.AutomationDataOptions\"U\n\x16\x43opySpecialRequestBody\x12;\n\x16\x61utomation_data_option\x18\x01 \x01(\x0e\x32\x1b.ptsl.AutomationDataOptions\"V\n\x17\x43learSpecialRequestBody\x12;\n\x16\x61utomation_data_option\x18\x01 \x01(\x0e\x32\x1b.ptsl.AutomationDataOptions\"R\n\x17PasteSpecialRequestBody\x12\x37\n\x14paste_special_option\x18\x01 \x01(\x0e\x32\x19.ptsl.PasteSpecialOptions\"G\n\rEM_SourceInfo\x12(\n\x0bsource_type\x18\x01 \x01(\x0e\x32\x13.ptsl.EM_SourceType\x12\x0c\n\x04name\x18\x02 \x01(\t\"\x97\x02\n\x0c\x45M_AudioInfo\x12/\n\x10\x63ompression_type\x18\x01 \x01(\x0e\x32\x15.ptsl.CompressionType\x12)\n\rexport_format\x18\x02 \x01(\x0e\x32\x12.ptsl.ExportFormat\x12!\n\tbit_depth\x18\x03 \x01(\x0e\x32\x0e.ptsl.BitDepth\x12%\n\x0bsample_rate\x18\x04 \x01(\x0e\x32\x10.ptsl.SampleRate\x12/\n\x15pad_to_frame_boundary\x18\x05 \x01(\x0e\x32\x10.ptsl.TripleBool\x12\x30\n\x0f\x64\x65livery_format\x18\x06 \x01(\x0e\x32\x17.ptsl.EM_DeliveryFormat\"]\n\x11PropertyContainer\x12\x16\n\x0e\x63ontainer_name\x18\x01 \x01(\t\x12!\n\x04type\x18\x02 \x01(\x0e\x32\x13.ptsl.DP_ValueTypes\x12\r\n\x05value\x18\x03 \x01(\t\"R\n\x0c\x45M_CodecInfo\x12\x12\n\ncodec_name\x18\x01 \x01(\t\x12.\n\rproperty_list\x18\x02 \x03(\x0b\x32\x17.ptsl.PropertyContainer\"\xc5\x01\n\x0c\x45M_VideoInfo\x12\'\n\rinclude_video\x18\x01 \x01(\x0e\x32\x10.ptsl.TripleBool\x12\x32\n\rexport_option\x18\x02 \x01(\x0e\x32\x1b.ptsl.EM_VideoExportOptions\x12\x30\n\x16replace_timecode_track\x18\x03 \x01(\x0e\x32\x10.ptsl.TripleBool\x12&\n\ncodec_info\x18\x04 \x01(\x0b\x32\x12.ptsl.EM_CodecInfo\"\xf3\x02\n\x10\x45M_ImportOptions\x12\x32\n\x12import_destination\x18\x01 \x01(\x0e\x32\x16.ptsl.MediaDestination\x12,\n\x0fimport_location\x18\x02 \x01(\x0e\x32\x13.ptsl.MediaLocation\x12\x1a\n\x12gaps_between_clips\x18\x03 \x01(\x05\x12\x30\n\x16import_audio_from_file\x18\x04 \x01(\x0e\x32\x10.ptsl.TripleBool\x12\x36\n\x1cremove_existing_video_tracks\x18\x05 \x01(\x0e\x32\x10.ptsl.TripleBool\x12\x35\n\x1bremove_existing_video_clips\x18\x06 \x01(\x0e\x32\x10.ptsl.TripleBool\x12@\n&clear_destination_video_track_playlist\x18\x07 \x01(\x0e\x32\x10.ptsl.TripleBool\"\xb7\x01\n\x0f\x45M_LocationInfo\x12-\n\x13import_after_bounce\x18\x01 \x01(\x0e\x32\x10.ptsl.TripleBool\x12.\n\x0eimport_options\x18\x02 \x01(\x0b\x32\x16.ptsl.EM_ImportOptions\x12\x32\n\x10\x66ile_destination\x18\x03 \x01(\x0e\x32\x18.ptsl.EM_FileDestination\x12\x11\n\tdirectory\x18\x04 \x01(\t\"\xa4\x01\n\x11\x45M_DolbyAtmosInfo\x12\x33\n\x19\x61\x64\x64_first_frame_of_action\x18\x01 \x01(\x0e\x32\x10.ptsl.TripleBool\x12\x16\n\x0etimecode_value\x18\x02 \x01(\t\x12\x12\n\nframe_rate\x18\x03 \x01(\x05\x12.\n\rproperty_list\x18\x04 \x03(\x0b\x32\x17.ptsl.PropertyContainer\"\x96\x03\n\x14\x45xportMixRequestBody\x12\x13\n\x0bpreset_path\x18\x01 \x01(\t\x12\x11\n\tfile_name\x18\x02 \x01(\t\x12$\n\tfile_type\x18\x03 \x01(\x0e\x32\x11.ptsl.EM_FileType\x12\'\n\nfiles_list\x18\x04 \x03(\x0b\x32\x13.ptsl.EM_SourceInfo\x12&\n\naudio_info\x18\x05 \x01(\x0b\x32\x12.ptsl.EM_AudioInfo\x12&\n\nvideo_info\x18\x06 \x01(\x0b\x32\x12.ptsl.EM_VideoInfo\x12,\n\rlocation_info\x18\x07 \x01(\x0b\x32\x15.ptsl.EM_LocationInfo\x12\x31\n\x10\x64olby_atmos_info\x18\x08 \x01(\x0b\x32\x17.ptsl.EM_DolbyAtmosInfo\x12(\n\x0eoffline_bounce\x18\t \x01(\x0e\x32\x10.ptsl.TripleBool\x12,\n\x0fmix_source_list\x18\n \x03(\x0b\x32\x13.ptsl.EM_SourceInfo\"\xd5\x01\n\x12PropertyDescriptor\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\'\n\nvalue_type\x18\x02 \x01(\x0e\x32\x13.ptsl.DP_ValueTypes\x12\x13\n\x0bobject_type\x18\x03 \x01(\t\x12\x10\n\x08required\x18\x04 \x01(\x08\x12\x13\n\x0b\x64\x65scription\x18\x05 \x01(\t\x12\r\n\x05units\x18\x06 \x01(\t\x12\x17\n\x0f\x61\x63\x63\x65pted_values\x18\x07 \x03(\t\x12\x11\n\tmax_value\x18\x08 \x01(\t\x12\x11\n\tmin_value\x18\t \x01(\t\"S\n\x1fGetDynamicPropertiesRequestBody\x12\x30\n\rproperty_type\x18\x01 \x01(\x0e\x32\x19.ptsl.DynamicPropertyType\"w\n\x19GetDynamicPropertiesGroup\x12)\n\x08key_list\x18\x01 \x03(\x0b\x32\x17.ptsl.PropertyContainer\x12/\n\rproperty_list\x18\x02 \x03(\x0b\x32\x18.ptsl.PropertyDescriptor\"\x89\x01\n GetDynamicPropertiesResponseBody\x12\x30\n\rproperty_type\x18\x01 \x01(\x0e\x32\x19.ptsl.DynamicPropertyType\x12\x33\n\ngroup_list\x18\x02 \x03(\x0b\x32\x1f.ptsl.GetDynamicPropertiesGroup\"x\n\x0fSpotRequestBody\x12\x36\n\x14track_offset_options\x18\x01 \x01(\x0e\x32\x18.ptsl.TrackOffsetOptions\x12-\n\rlocation_data\x18\x02 \x01(\x0b\x32\x16.ptsl.SpotLocationData\"\xf5\x03\n\"ExportSessionInfoAsTextRequestBody\x12\x19\n\x11include_file_list\x18\x01 \x01(\x08\x12\x19\n\x11include_clip_list\x18\x02 \x01(\x08\x12\x17\n\x0finclude_markers\x18\x03 \x01(\x08\x12\x1b\n\x13include_plugin_list\x18\x04 \x01(\x08\x12\x1a\n\x12include_track_edls\x18\x05 \x01(\x08\x12\x17\n\x0fshow_sub_frames\x18\x06 \x01(\x08\x12\x1f\n\x17include_user_timestamps\x18\x07 \x01(\x08\x12,\n\x0ftrack_list_type\x18\x08 \x01(\x0e\x32\x13.ptsl.TrackListType\x12\x32\n\x12\x66\x61\x64\x65_handling_type\x18\t \x01(\x0e\x32\x16.ptsl.FadeHandlingType\x12\x36\n\x14track_offset_options\x18\n \x01(\x0e\x32\x18.ptsl.TrackOffsetOptions\x12\x33\n\x13text_as_file_format\x18\x0b \x01(\x0e\x32\x16.ptsl.TextAsFileFormat\x12)\n\x0boutput_type\x18\x0c \x01(\x0e\x32\x14.ptsl.ESI_OutputType\x12\x13\n\x0boutput_path\x18\r \x01(\t\";\n#ExportSessionInfoAsTextResponseBody\x12\x14\n\x0csession_info\x18\x01 \x01(\t\"J\n\x1aSetPlaybackModeRequestBody\x12,\n\rplayback_mode\x18\x01 \x01(\x0e\x32\x15.ptsl.PM_PlaybackMode\"\x86\x01\n\x1bSetPlaybackModeResponseBody\x12\x34\n\x15\x63urrent_playback_mode\x18\x01 \x01(\x0e\x32\x15.ptsl.PM_PlaybackMode\x12\x31\n\x12playback_mode_list\x18\x02 \x03(\x0e\x32\x15.ptsl.PM_PlaybackMode\"b\n\x18SetRecordModeRequestBody\x12(\n\x0brecord_mode\x18\x01 \x01(\x0e\x32\x13.ptsl.RM_RecordMode\x12\x1c\n\x14record_arm_transport\x18\x02 \x01(\x08\"|\n\x19SetRecordModeResponseBody\x12\x30\n\x13\x63urrent_record_mode\x18\x01 \x01(\x0e\x32\x13.ptsl.RM_RecordMode\x12-\n\x10record_mode_list\x18\x02 \x03(\x0e\x32\x13.ptsl.RM_RecordMode\"\x8b\x01\n!GetSessionAudioFormatResponseBody\x12\x31\n\x0f\x63urrent_setting\x18\x01 \x01(\x0e\x32\x18.ptsl.SessionAudioFormat\x12\x33\n\x11possible_settings\x18\x02 \x03(\x0e\x32\x18.ptsl.SessionAudioFormat\"I\n GetSessionSampleRateResponseBody\x12%\n\x0bsample_rate\x18\x01 \x01(\x0e\x32\x10.ptsl.SampleRate\"t\n\x1eGetSessionBitDepthResponseBody\x12\'\n\x0f\x63urrent_setting\x18\x01 \x01(\x0e\x32\x0e.ptsl.BitDepth\x12)\n\x11possible_settings\x18\x02 \x03(\x0e\x32\x0e.ptsl.BitDepth\"\\\n&GetSessionInterleavedStateResponseBody\x12\x17\n\x0f\x63urrent_setting\x18\x01 \x01(\x08\x12\x19\n\x11possible_settings\x18\x02 \x03(\x08\"\x8e\x01\n\"GetSessionTimeCodeRateResponseBody\x12\x32\n\x0f\x63urrent_setting\x18\x01 \x01(\x0e\x32\x19.ptsl.SessionTimeCodeRate\x12\x34\n\x11possible_settings\x18\x02 \x03(\x0e\x32\x19.ptsl.SessionTimeCodeRate\"\x94\x01\n$GetSessionFeetFramesRateResponseBody\x12\x34\n\x0f\x63urrent_setting\x18\x01 \x01(\x0e\x32\x1b.ptsl.SessionFeetFramesRate\x12\x36\n\x11possible_settings\x18\x02 \x03(\x0e\x32\x1b.ptsl.SessionFeetFramesRate\"\x8f\x01\n+GetSessionAudioRatePullSettingsResponseBody\x12.\n\x0f\x63urrent_setting\x18\x01 \x01(\x0e\x32\x15.ptsl.SessionRatePull\x12\x30\n\x11possible_settings\x18\x02 \x03(\x0e\x32\x15.ptsl.SessionRatePull\"\x8f\x01\n+GetSessionVideoRatePullSettingsResponseBody\x12.\n\x0f\x63urrent_setting\x18\x01 \x01(\x0e\x32\x15.ptsl.SessionRatePull\x12\x30\n\x11possible_settings\x18\x02 \x03(\x0e\x32\x15.ptsl.SessionRatePull\"2\n\x1aGetSessionNameResponseBody\x12\x14\n\x0csession_name\x18\x01 \x01(\t\"F\n\x1aGetSessionPathResponseBody\x12(\n\x0csession_path\x18\x01 \x01(\x0b\x32\x12.ptsl.FileLocation\"=\n\x1fGetSessionStartTimeResponseBody\x12\x1a\n\x12session_start_time\x18\x01 \x01(\t\"6\n\x1cGetSessionLengthResponseBody\x12\x16\n\x0esession_length\x18\x01 \x01(\t\"R\n SetSessionAudioFormatRequestBody\x12.\n\x0c\x61udio_format\x18\x01 \x01(\x0e\x32\x18.ptsl.SessionAudioFormat\"B\n\x1dSetSessionBitDepthRequestBody\x12!\n\tbit_depth\x18\x01 \x01(\x0e\x32\x0e.ptsl.BitDepth\"B\n%SetSessionInterleavedStateRequestBody\x12\x19\n\x11interleaved_state\x18\x01 \x01(\x08\"V\n!SetSessionTimeCodeRateRequestBody\x12\x31\n\x0etime_code_rate\x18\x01 \x01(\x0e\x32\x19.ptsl.SessionTimeCodeRate\"\\\n#SetSessionFeetFramesRateRequestBody\x12\x35\n\x10\x66\x65\x65t_frames_rate\x18\x01 \x01(\x0e\x32\x1b.ptsl.SessionFeetFramesRate\"\\\n*SetSessionAudioRatePullSettingsRequestBody\x12.\n\x0f\x61udio_rate_pull\x18\x01 \x01(\x0e\x32\x15.ptsl.SessionRatePull\"\\\n*SetSessionVideoRatePullSettingsRequestBody\x12.\n\x0fvideo_rate_pull\x18\x01 \x01(\x0e\x32\x15.ptsl.SessionRatePull\"\x95\x01\n\x1eSetSessionStartTimeRequestBody\x12\x1a\n\x12session_start_time\x18\x01 \x01(\t\x12\x33\n\x11track_offset_opts\x18\x02 \x01(\x0e\x32\x18.ptsl.TrackOffsetOptions\x12\"\n\x1amaintain_relative_position\x18\x03 \x01(\x08\"5\n\x1bSetSessionLengthRequestBody\x12\x16\n\x0esession_length\x18\x01 \x01(\t\"-\n\x1aGetPTSLVersionResponseBody\x12\x0f\n\x07version\x18\x01 \x01(\x05\"\x80\x01\n\x1bGetPlaybackModeResponseBody\x12/\n\x10\x63urrent_settings\x18\x01 \x03(\x0e\x32\x15.ptsl.PM_PlaybackMode\x12\x30\n\x11possible_settings\x18\x02 \x03(\x0e\x32\x15.ptsl.PM_PlaybackMode\"y\n\x19GetRecordModeResponseBody\x12,\n\x0f\x63urrent_setting\x18\x01 \x01(\x0e\x32\x13.ptsl.RM_RecordMode\x12.\n\x11possible_settings\x18\x02 \x03(\x0e\x32\x13.ptsl.RM_RecordMode\"\x85\x01\n\x1dGetTransportStateResponseBody\x12\x30\n\x0f\x63urrent_setting\x18\x01 \x01(\x0e\x32\x17.ptsl.TS_TransportState\x12\x32\n\x11possible_settings\x18\x02 \x03(\x0e\x32\x17.ptsl.TS_TransportState\";\n\x1dGetTransportArmedResponseBody\x12\x1a\n\x12is_transport_armed\x18\x01 \x01(\x08\"7\n\x1e\x43learMemoryLocationRequestBody\x12\x15\n\rlocation_list\x18\x01 \x03(\x05\"5\n\x1e\x41uthorizeConnectionRequestBody\x12\x13\n\x0b\x61uth_string\x18\x01 \x01(\t\"]\n\x1f\x41uthorizeConnectionResponseBody\x12\x15\n\ris_authorized\x18\x01 \x01(\x08\x12\x0f\n\x07message\x18\x02 \x01(\t\x12\x12\n\nsession_id\x18\x03 \x01(\t\"t\n\x1dRenameSelectedClipRequestBody\x12,\n\rclip_location\x18\x01 \x01(\x0e\x32\x15.ptsl.CL_ClipLocation\x12\x10\n\x08new_name\x18\x02 \x01(\t\x12\x13\n\x0brename_file\x18\x03 \x01(\x08\"W\n\x1bRenameTargetClipRequestBody\x12\x11\n\tclip_name\x18\x01 \x01(\t\x12\x10\n\x08new_name\x18\x02 \x01(\t\x12\x13\n\x0brename_file\x18\x03 \x01(\x08\"\x83\x02\n\x0eMemoryLocation\x12\x0e\n\x06number\x18\x01 \x01(\x05\x12\x0c\n\x04name\x18\x02 \x01(\t\x12\x12\n\nstart_time\x18\x03 \x01(\t\x12\x10\n\x08\x65nd_time\x18\x04 \x01(\t\x12-\n\x0ftime_properties\x18\x05 \x01(\x0e\x32\x14.ptsl.TimeProperties\x12\x30\n\treference\x18\x06 \x01(\x0e\x32\x1d.ptsl.MemoryLocationReference\x12:\n\x12general_properties\x18\x07 \x01(\x0b\x32\x1e.ptsl.MemoryLocationProperties\x12\x10\n\x08\x63omments\x18\x08 \x01(\t\"\xfb\x01\n\x18MemoryLocationProperties\x12\x15\n\rzoom_settings\x18\x01 \x01(\x08\x12\x1b\n\x13pre_post_roll_times\x18\x02 \x01(\x08\x12\x18\n\x10track_visibility\x18\x03 \x01(\x08\x12\x15\n\rtrack_heights\x18\x04 \x01(\x08\x12\x15\n\rgroup_enables\x18\x05 \x01(\x08\x12\x1c\n\x14window_configuration\x18\x06 \x01(\x08\x12\"\n\x1awindow_configuration_index\x18\x07 \x01(\x05\x12!\n\x19window_configuration_name\x18\x08 \x01(\t\"\x92\x02\n\x1d\x45\x64itMemoryLocationRequestBody\x12\x0e\n\x06number\x18\x01 \x01(\x05\x12\x0c\n\x04name\x18\x02 \x01(\t\x12\x12\n\nstart_time\x18\x03 \x01(\t\x12\x10\n\x08\x65nd_time\x18\x04 \x01(\t\x12-\n\x0ftime_properties\x18\x05 \x01(\x0e\x32\x14.ptsl.TimeProperties\x12\x30\n\treference\x18\x06 \x01(\x0e\x32\x1d.ptsl.MemoryLocationReference\x12:\n\x12general_properties\x18\x07 \x01(\x0b\x32\x1e.ptsl.MemoryLocationProperties\x12\x10\n\x08\x63omments\x18\x08 \x01(\t\" \n\x1e\x45\x64itMemoryLocationResponseBody\"\x94\x02\n\x1f\x43reateMemoryLocationRequestBody\x12\x0e\n\x06number\x18\x01 \x01(\x05\x12\x0c\n\x04name\x18\x02 \x01(\t\x12\x12\n\nstart_time\x18\x03 \x01(\t\x12\x10\n\x08\x65nd_time\x18\x04 \x01(\t\x12-\n\x0ftime_properties\x18\x05 \x01(\x0e\x32\x14.ptsl.TimeProperties\x12\x30\n\treference\x18\x06 \x01(\x0e\x32\x1d.ptsl.MemoryLocationReference\x12:\n\x12general_properties\x18\x07 \x01(\x0b\x32\x1e.ptsl.MemoryLocationProperties\x12\x10\n\x08\x63omments\x18\x08 \x01(\t\"\"\n CreateMemoryLocationResponseBody\"\x1f\n\x1dGetMemoryLocationsRequestBody\"P\n\x1eGetMemoryLocationsResponseBody\x12.\n\x10memory_locations\x18\x01 \x03(\x0b\x32\x14.ptsl.MemoryLocation\"O\n\x1dRegisterConnectionRequestBody\x12\x14\n\x0c\x63ompany_name\x18\x01 \x01(\t\x12\x18\n\x10\x61pplication_name\x18\x02 \x01(\t\"4\n\x1eRegisterConnectionResponseBody\x12\x12\n\nsession_id\x18\x01 \x01(\t*\x8a\r\n\tCommandId\x12\x11\n\rCreateSession\x10\x00\x12\x0f\n\x0bOpenSession\x10\x01\x12\n\n\x06Import\x10\x02\x12\x10\n\x0cGetTrackList\x10\x03\x12\x19\n\x15SelectAllClipsOnTrack\x10\x04\x12!\n\x1d\x45xtendSelectionToTargetTracks\x10\x05\x12\x13\n\x0fTrimToSelection\x10\x06\x12\x1c\n\x18\x43reateFadesBasedOnPreset\x10\x07\x12\x15\n\x11RenameTargetTrack\x10\x08\x12\x13\n\x0f\x43onsolidateClip\x10\t\x12\x16\n\x12\x45xportClipsAsFiles\x10\n\x12 \n\x1c\x45xportSelectedTracksAsAAFOMF\x10\x0b\x12\x11\n\rGetTaskStatus\x10\x0c\x12\x12\n\x0eHostReadyCheck\x10\r\x12\x1b\n\x17RefreshTargetAudioFiles\x10\x0e\x12 \n\x1cRefreshAllModifiedAudioFiles\x10\x0f\x12\x13\n\x0fGetFileLocation\x10\x10\x12\x10\n\x0c\x43loseSession\x10\x11\x12\x0f\n\x0bSaveSession\x10\x12\x12\x11\n\rSaveSessionAs\x10\x13\x12\x07\n\x03\x43ut\x10\x14\x12\x08\n\x04\x43opy\x10\x15\x12\t\n\x05Paste\x10\x16\x12\t\n\x05\x43lear\x10\x17\x12\x0e\n\nCutSpecial\x10\x18\x12\x0f\n\x0b\x43opySpecial\x10\x19\x12\x10\n\x0c\x43learSpecial\x10\x1a\x12\x10\n\x0cPasteSpecial\x10\x1b\x12\r\n\tExportMix\x10\x1c\x12\x08\n\x04Spot\x10\x1d\x12\x1b\n\x17\x45xportSessionInfoAsText\x10\x1e\x12\x18\n\x14GetDynamicProperties\x10\x1f\x12\x13\n\x0fSetPlaybackMode\x10 \x12\x11\n\rSetRecordMode\x10!\x12\x19\n\x15GetSessionAudioFormat\x10\"\x12\x18\n\x14GetSessionSampleRate\x10#\x12\x16\n\x12GetSessionBitDepth\x10$\x12\x1e\n\x1aGetSessionInterleavedState\x10%\x12\x1a\n\x16GetSessionTimeCodeRate\x10&\x12\x1c\n\x18GetSessionFeetFramesRate\x10\'\x12#\n\x1fGetSessionAudioRatePullSettings\x10(\x12#\n\x1fGetSessionVideoRatePullSettings\x10)\x12\x12\n\x0eGetSessionName\x10*\x12\x12\n\x0eGetSessionPath\x10+\x12\x17\n\x13GetSessionStartTime\x10,\x12\x14\n\x10GetSessionLength\x10-\x12\x19\n\x15SetSessionAudioFormat\x10.\x12\x16\n\x12SetSessionBitDepth\x10/\x12\x1e\n\x1aSetSessionInterleavedState\x10\x30\x12\x1a\n\x16SetSessionTimeCodeRate\x10\x31\x12\x1c\n\x18SetSessionFeetFramesRate\x10\x32\x12#\n\x1fSetSessionAudioRatePullSettings\x10\x33\x12#\n\x1fSetSessionVideoRatePullSettings\x10\x34\x12\x17\n\x13SetSessionStartTime\x10\x35\x12\x14\n\x10SetSessionLength\x10\x36\x12\x12\n\x0eGetPTSLVersion\x10\x37\x12\x13\n\x0fGetPlaybackMode\x10\x38\x12\x11\n\rGetRecordMode\x10\x39\x12\x15\n\x11GetTransportArmed\x10:\x12\x15\n\x11GetTransportState\x10;\x12\x17\n\x13\x41uthorizeConnection\x10<\x12\x17\n\x13\x43learMemoryLocation\x10=\x12\x16\n\x12RenameSelectedClip\x10>\x12\x14\n\x10RenameTargetClip\x10?\x12\x13\n\x0fTogglePlayState\x10@\x12\x16\n\x12ToggleRecordEnable\x10\x41\x12\x11\n\rPlayHalfSpeed\x10\x42\x12\x13\n\x0fRecordHalfSpeed\x10\x43\x12\x16\n\x12\x45\x64itMemoryLocation\x10\x44\x12\x16\n\x12GetMemoryLocations\x10\x45\x12\x16\n\x12RegisterConnection\x10\x46\x12\x18\n\x14\x43reateMemoryLocation\x10G*\xa7\x01\n\nTaskStatus\x12\n\n\x06Queued\x10\x00\x12\x0b\n\x07Pending\x10\x01\x12\x0e\n\nInProgress\x10\x02\x12\r\n\tCompleted\x10\x03\x12\n\n\x06\x46\x61iled\x10\x04\x12\x17\n\x13WaitingForUserInput\x10\x05\x12\x1c\n\x18\x43ompletedWithBadResponse\x10\x06\x12\x1e\n\x1a\x46\x61iledWithBadErrorResponse\x10\x07*\xed\x07\n\x10\x43ommandErrorType\x12\x17\n\x13OS_WritePermissions\x10\x00\x12\x10\n\x0cOS_ErrorCode\x10\x01\x12\x16\n\x12OS_NoLocationFound\x10\x02\x12\x15\n\x11OS_NoSessionFound\x10\x03\x12\x17\n\x13OS_FilePathLocation\x10\x04\x12\x10\n\x0cOS_ReadError\x10\x05\x12\x10\n\x0cOS_DiskSpace\x10\x06\x12\x14\n\x10OS_DuplicateName\x10\x07\x12\x18\n\x14OS_IllegalCharacters\x10\x08\x12\x16\n\x12OS_CharactersLimit\x10\t\x12\x1d\n\x19OS_ProToolsIsNotAvailable\x10\n\x12\x16\n\x12OS_NoFilePathFound\x10\x0b\x12\x13\n\x0fPT_UnknownError\x10\x64\x12\x16\n\x12PT_NoTemplateGroup\x10\x65\x12\x11\n\rPT_NoTemplate\x10\x66\x12\x19\n\x15PT_SampleRateMismatch\x10g\x12\x18\n\x14PT_NoVideoTrackFound\x10h\x12\x14\n\x10PT_NoTracksFound\x10i\x12\x16\n\x12PT_NoOpenedSession\x10j\x12\x13\n\x0fPT_NoTrackFound\x10k\x12\x13\n\x0fPT_NoClipsFound\x10l\x12\x12\n\x0ePT_NoSelection\x10m\x12\x12\n\x0ePT_RecordDrive\x10n\x12\x14\n\x10PT_NoPresetFound\x10o\x12\x12\n\x0ePT_FileTypeMXF\x10p\x12\x15\n\x11PT_CopyOptionCopy\x10q\x12\x15\n\x11PT_CopyOptionLink\x10r\x12\x14\n\x10PT_QuantizeEdits\x10s\x12\x1b\n\x17PT_ExportAsMultichannel\x10t\x12 \n\x1cPT_IllegalCharactersComments\x10u\x12$\n PT_IllegalCharactersSequenceName\x10v\x12\x1c\n\x18PT_MaxCharactersComments\x10w\x12 \n\x1cPT_MaxCharactersSequenceName\x10x\x12\x15\n\x11PT_NoSequenceName\x10y\x12\x12\n\x0ePT_InvalidTask\x10z\x12\x13\n\x0fPT_FileNotFound\x10{\x12\x17\n\x13PT_InvalidSelection\x10|\x12\x16\n\x12PT_ReadOnlySession\x10}\x12\x17\n\x13PT_InvalidParameter\x10~\x12\x18\n\x13SDK_VersionMismatch\x10\x91\x03\x12\x17\n\x12SDK_NotImplemented\x10\x92\x03*\xf3\x01\n\tTrackType\x12\x0b\n\x07Unknown\x10\x00\x12\x08\n\x04Midi\x10\x01\x12\x0e\n\nAudioTrack\x10\x02\x12\x07\n\x03\x41ux\x10\x03\x12\x0e\n\nVideoTrack\x10\x04\x12\x07\n\x03Vca\x10\x05\x12\t\n\x05Tempo\x10\x06\x12\x0b\n\x07Markers\x10\x07\x12\t\n\x05Meter\x10\x08\x12\x10\n\x0cKeySignature\x10\t\x12\x10\n\x0c\x43hordSymbols\x10\n\x12\x0e\n\nInstrument\x10\x0b\x12\n\n\x06Master\x10\x0c\x12\x08\n\x04Heat\x10\r\x12\x0f\n\x0b\x42\x61sicFolder\x10\x0e\x12\x11\n\rRoutingFolder\x10\x0f\x12\x0c\n\x08\x43ompLane\x10\x10*e\n\x13TrackAttributeState\x12\x08\n\x04None\x10\x00\x12\x11\n\rSetExplicitly\x10\x01\x12\x11\n\rSetImplicitly\x10\x02\x12\x1e\n\x1aSetExplicitlyAndImplicitly\x10\x03*<\n\x08\x46ileType\x12\x0b\n\x07\x46T_WAVE\x10\x00\x12\x0b\n\x07\x46T_AIFF\x10\x01\x12\n\n\x06\x46T_AAF\x10\x02\x12\n\n\x06\x46T_OMF\x10\x03*\x82\x01\n\nIOSettings\x12\x0b\n\x07IO_None\x10\x00\x12\x0b\n\x07IO_Last\x10\x01\x12\x10\n\x0cIO_StereoMix\x10\x02\x12\x10\n\x0cIO_51FilmMix\x10\x03\x12\x11\n\rIO_51SMPTEMix\x10\x04\x12\x0f\n\x0bIO_51DTSMix\x10\x05\x12\x12\n\x0eIO_UserDefined\x10\x06*I\n\nImportType\x12\x0b\n\x07Session\x10\x00\x12\t\n\x05\x41udio\x10\x01\x12\t\n\x05Video\x10\x02\x12\x08\n\x04MIDI\x10\x03\x12\x0e\n\nClipGroups\x10\x04*\x83\x01\n\x11\x41udioMediaOptions\x12\x15\n\x11LinkToSourceAudio\x10\x00\x12\x17\n\x13\x43opyFromSourceAudio\x10\x01\x12\x1e\n\x1a\x43onsolidateFromSourceAudio\x10\x02\x12\x1e\n\x1a\x46orceToTargetSessionFormat\x10\x03*f\n\x11VideoMediaOptions\x12\x15\n\x11LinkToSourceVideo\x10\x00\x12\x17\n\x13\x43opyFromSourceVideo\x10\x01\x12!\n\x1dImportAsOfflineSatelliteMedia\x10\x02*M\n\x11MatchTrackOptions\x12\x0b\n\x07MT_None\x10\x00\x12\x12\n\x0eMT_MatchTracks\x10\x01\x12\x17\n\x13MT_ImportAsNewTrack\x10\x02*x\n\x16TimeCodeMappingOptions\x12\"\n\x1eMaintainAbsoluteTimeCodeValues\x10\x00\x12\"\n\x1eMaintainRelativeTimeCodeValues\x10\x01\x12\x16\n\x12MapStartTimeCodeTo\x10\x02*[\n\x12TrackOffsetOptions\x12\r\n\tBarsBeats\x10\x00\x12\x0b\n\x07MinSecs\x10\x01\x12\x0c\n\x08TimeCode\x10\x02\x12\x0e\n\nFeetFrames\x10\x03\x12\x0b\n\x07Samples\x10\x04*K\n\x11\x43onversionQuality\x12\x07\n\x03Low\x10\x00\x12\x08\n\x04Good\x10\x01\x12\n\n\x06\x42\x65tter\x10\x02\x12\x08\n\x04\x42\x65st\x10\x03\x12\r\n\tTweakHead\x10\x04*s\n\x13MainPlaylistOptions\x12\"\n\x1eImportReplaceExistingPlaylists\x10\x00\x12\'\n#ImportOverlayNewOnExistingPlaylists\x10\x01\x12\x0f\n\x0b\x44oNotImport\x10\x02*M\n\x0f\x41udioOperations\x12\x0c\n\x08\x41\x64\x64\x41udio\x10\x00\x12\r\n\tCopyAudio\x10\x01\x12\x10\n\x0c\x43onvertAudio\x10\x02\x12\x0b\n\x07\x44\x65\x66\x61ult\x10\x03*X\n\x10MediaDestination\x12\x0b\n\x07MD_None\x10\x00\x12\x15\n\x11MD_MainVideoTrack\x10\x01\x12\x0f\n\x0bMD_NewTrack\x10\x02\x12\x0f\n\x0bMD_ClipList\x10\x03*b\n\rMediaLocation\x12\x0b\n\x07ML_None\x10\x00\x12\x13\n\x0fML_SessionStart\x10\x01\x12\x10\n\x0cML_SongStart\x10\x02\x12\x10\n\x0cML_Selection\x10\x03\x12\x0b\n\x07ML_Spot\x10\x04*\xc6\x02\n\x0fTrackListFilter\x12\x07\n\x03\x41ll\x10\x00\x12\x0c\n\x08Selected\x10\x01\x12\x16\n\x12SelectedExplicitly\x10\x02\x12\x16\n\x12SelectedImplicitly\x10\x03\x12\x1b\n\x17WithClipsOnMainPlaylist\x10\x04\x12 \n\x1cWithAutomationOnMainPlaylist\x10\x05\x12\x0c\n\x08Inactive\x10\x06\x12\x16\n\x12InactiveExplicitly\x10\x07\x12\x16\n\x12InactiveImplicitly\x10\x08\x12\n\n\x06Hidden\x10\t\x12\x14\n\x10HiddenExplicitly\x10\n\x12\x14\n\x10HiddenImplicitly\x10\x0b\x12\n\n\x06Locked\x10\x0c\x12\t\n\x05Muted\x10\r\x12\n\n\x06\x46rozen\x10\x0e\x12\x08\n\x04Open\x10\x0f\x12\n\n\x06Online\x10\x10*5\n\x10SpotLocationType\x12\t\n\x05Start\x10\x00\x12\r\n\tSyncPoint\x10\x01\x12\x07\n\x03\x45nd\x10\x02*Q\n\x0c\x45xportFormat\x12\x0b\n\x07\x45\x46_None\x10\x00\x12\x0b\n\x07\x45\x46_Mono\x10\x01\x12\x13\n\x0f\x45\x46_MultipleMono\x10\x02\x12\x12\n\x0e\x45\x46_Interleaved\x10\x03*D\n\x0e\x45xportFileType\x12\x07\n\x03WAV\x10\x00\x12\x08\n\x04\x41IFF\x10\x01\x12\x07\n\x03MXF\x10\x02\x12\x07\n\x03MP3\x10\x03\x12\r\n\tQuickTime\x10\x04*>\n\x08\x42itDepth\x12\x0c\n\x08\x42it_None\x10\x00\x12\t\n\x05\x42it16\x10\x01\x12\t\n\x05\x42it24\x10\x02\x12\x0e\n\nBit32Float\x10\x03*F\n\x17ResolveDuplicateNamesBy\x12\x10\n\x0c\x41utoRenaming\x10\x00\x12\x19\n\x15ReplacingWithNewFiles\x10\x01*O\n\x13\x45xportAsAAFFileType\x12\x0b\n\x07\x41\x41\x46_WAV\x10\x00\x12\x0c\n\x08\x41\x41\x46_AIFF\x10\x01\x12\x0b\n\x07\x41\x41\x46_MXF\x10\x02\x12\x10\n\x0c\x41\x41\x46_Embedded\x10\x03*/\n\x0f\x41\x41\x46\x46ileBitDepth\x12\r\n\tAAF_Bit16\x10\x00\x12\r\n\tAAF_Bit24\x10\x01*^\n\nCopyOption\x12\x1e\n\x1a\x43onsolidateFromSourceMedia\x10\x00\x12\x17\n\x13\x43opyFromSourceMedia\x10\x01\x12\x17\n\x13LinkFromSourceMedia\x10\x02*\xe8\x01\n\x16\x46ileLocationTypeFilter\x12\r\n\tAll_Files\x10\x00\x12\x14\n\x10OnTimeline_Files\x10\x01\x12\x17\n\x13NotOnTimeline_Files\x10\x02\x12\x10\n\x0cOnline_Files\x10\x03\x12\x11\n\rOffline_Files\x10\x04\x12\x0f\n\x0b\x41udio_Files\x10\x05\x12\x0f\n\x0bVideo_Files\x10\x06\x12\x12\n\x0eRendered_Files\x10\x07\x12\x19\n\x15SelectedClipsTimeline\x10\x65\x12\x1a\n\x16SelectedClipsClipsList\x10\x66*w\n\x15\x41utomationDataOptions\x12\x12\n\x0e\x41ll_Automation\x10\x00\x12\x12\n\x0ePan_Automation\x10\x01\x12\x15\n\x11PlugIn_Automation\x10\x02\x12\r\n\tClip_Gain\x10\x03\x12\x10\n\x0c\x43lip_Effects\x10\x04*^\n\x13PasteSpecialOptions\x12\t\n\x05Merge\x10\x00\x12\x1c\n\x18Repeat_To_Fill_Selection\x10\x01\x12\x1e\n\x1aTo_Current_Automation_Type\x10\x02*4\n\nTripleBool\x12\x0b\n\x07TB_None\x10\x00\x12\x0c\n\x08TB_False\x10\x01\x12\x0b\n\x07TB_True\x10\x02*5\n\rEM_SourceType\x12\x0f\n\x0bPhysicalOut\x10\x00\x12\x07\n\x03\x42us\x10\x01\x12\n\n\x06Output\x10\x02**\n\x0f\x43ompressionType\x12\x0b\n\x07\x43T_None\x10\x00\x12\n\n\x06\x43T_PCM\x10\x01*o\n\nSampleRate\x12\x0b\n\x07SR_None\x10\x00\x12\x0c\n\x08SR_44100\x10\x01\x12\x0c\n\x08SR_48000\x10\x02\x12\x0c\n\x08SR_96000\x10\x03\x12\r\n\tSR_176400\x10\x04\x12\r\n\tSR_192000\x10\x05\x12\x0c\n\x08SR_88200\x10\x06*K\n\x15\x45M_VideoExportOptions\x12\x0b\n\x07VE_None\x10\x00\x12\x13\n\x0fVE_SameAsSource\x10\x01\x12\x10\n\x0cVE_Transcode\x10\x02*l\n\x0b\x45M_FileType\x12\x0b\n\x07\x45M_None\x10\x00\x12\n\n\x06\x45M_MOV\x10\x01\x12\n\n\x06\x45M_WAV\x10\x02\x12\x0b\n\x07\x45M_AIFF\x10\x03\x12\n\n\x06\x45M_MP3\x10\x04\x12\x10\n\x0c\x45M_MXFOPAtom\x10\x05\x12\r\n\tEM_WAVADM\x10\x06*R\n\x12\x45M_FileDestination\x12\x0e\n\nEM_FD_None\x10\x00\x12\x17\n\x13\x45M_FD_SessionFolder\x10\x01\x12\x13\n\x0f\x45M_FD_Directory\x10\x02*U\n\x11\x45M_DeliveryFormat\x12\x0e\n\nEM_DF_None\x10\x00\x12\x1a\n\x16\x45M_DF_FilePerMixSource\x10\x01\x12\x14\n\x10\x45M_DF_SingleFile\x10\x02*\xbf\x02\n\rDP_ValueTypes\x12\x10\n\x0c\x44P_VT_double\x10\x00\x12\x0f\n\x0b\x44P_VT_float\x10\x01\x12\x0f\n\x0b\x44P_VT_int32\x10\x02\x12\x0f\n\x0b\x44P_VT_int64\x10\x03\x12\x10\n\x0c\x44P_VT_uint32\x10\x04\x12\x10\n\x0c\x44P_VT_uint64\x10\x05\x12\x10\n\x0c\x44P_VT_sint32\x10\x06\x12\x10\n\x0c\x44P_VT_sint64\x10\x07\x12\x11\n\rDP_VT_fixed32\x10\x08\x12\x11\n\rDP_VT_fixed64\x10\t\x12\x12\n\x0e\x44P_VT_sfixed32\x10\n\x12\x12\n\x0e\x44P_VT_sfixed64\x10\x0b\x12\x0e\n\nDP_VT_bool\x10\x0c\x12\x10\n\x0c\x44P_VT_string\x10\r\x12\x0f\n\x0b\x44P_VT_bytes\x10\x0e\x12\x0e\n\nDP_VT_enum\x10\x0f\x12\x10\n\x0c\x44P_VT_object\x10\x10*D\n\x13\x44ynamicPropertyType\x12\x13\n\x0f\x44P_EM_CodecInfo\x10\x00\x12\x18\n\x14\x44P_EM_DolbyAtmosInfo\x10\x01*6\n\rTrackListType\x12\r\n\tAllTracks\x10\x00\x12\x16\n\x12SelectedTracksOnly\x10\x01*Z\n\x10\x46\x61\x64\x65HandlingType\x12\x12\n\x0eShowCrossfades\x10\x00\x12\x16\n\x12\x44ontShowCrossfades\x10\x01\x12\x1a\n\x16\x43ombineCrossfadedClips\x10\x02**\n\x10TextAsFileFormat\x12\x0c\n\x08TextEdit\x10\x00\x12\x08\n\x04UTF8\x10\x01*.\n\x0e\x45SI_OutputType\x12\x0c\n\x08\x45SI_File\x10\x00\x12\x0e\n\nESI_String\x10\x01*F\n\x0fPM_PlaybackMode\x12\r\n\tPM_Normal\x10\x00\x12\x0b\n\x07PM_Loop\x10\x01\x12\x17\n\x13PM_DynamicTransport\x10\x02*~\n\rRM_RecordMode\x12\r\n\tRM_Normal\x10\x00\x12\x0b\n\x07RM_Loop\x10\x01\x12\x12\n\x0eRM_Destructive\x10\x02\x12\x11\n\rRM_QuickPunch\x10\x03\x12\x11\n\rRM_TrackPunch\x10\x04\x12\x17\n\x13RM_DestructivePunch\x10\x05*0\n\x12SessionAudioFormat\x12\x0c\n\x08SAF_WAVE\x10\x00\x12\x0c\n\x08SAF_AIFF\x10\x01*\xf4\x02\n\x13SessionTimeCodeRate\x12\x11\n\rSTCR_Fps23976\x10\x00\x12\x0e\n\nSTCR_Fps24\x10\x01\x12\x0e\n\nSTCR_Fps25\x10\x02\x12\x10\n\x0cSTCR_Fps2997\x10\x03\x12\x14\n\x10STCR_Fps2997Drop\x10\x04\x12\x0e\n\nSTCR_Fps30\x10\x05\x12\x12\n\x0eSTCR_Fps30Drop\x10\x06\x12\x11\n\rSTCR_Fps47952\x10\x07\x12\x0e\n\nSTCR_Fps48\x10\x08\x12\x0e\n\nSTCR_Fps50\x10\t\x12\x10\n\x0cSTCR_Fps5994\x10\n\x12\x14\n\x10STCR_Fps5994Drop\x10\x0b\x12\x0e\n\nSTCR_Fps60\x10\x0c\x12\x12\n\x0eSTCR_Fps60Drop\x10\r\x12\x0f\n\x0bSTCR_Fps100\x10\x0e\x12\x11\n\rSTCR_Fps11988\x10\x0f\x12\x15\n\x11STCR_Fps11988Drop\x10\x10\x12\x0f\n\x0bSTCR_Fps120\x10\x11\x12\x13\n\x0fSTCR_Fps120Drop\x10\x12*J\n\x15SessionFeetFramesRate\x12\x11\n\rSFFR_Fps23976\x10\x00\x12\x0e\n\nSFFR_Fps24\x10\x01\x12\x0e\n\nSFFR_Fps25\x10\x02*\xa5\x01\n\x0fSessionRatePull\x12\x0c\n\x08SRP_None\x10\x00\x12\x0c\n\x08SRP_Up01\x10\x01\x12\x0e\n\nSRP_Down01\x10\x02\x12\x0b\n\x07SRP_Up4\x10\x03\x12\x0f\n\x0bSRP_Up4Up01\x10\x04\x12\x11\n\rSRP_Up4Down01\x10\x05\x12\r\n\tSRP_Down4\x10\x06\x12\x11\n\rSRP_Down4Up01\x10\x07\x12\x13\n\x0fSRP_Down4Down01\x10\x08*\xb1\x03\n\x11TS_TransportState\x12\x17\n\x13TS_TransportPlaying\x10\x00\x12\x17\n\x13TS_TransportStopped\x10\x01\x12\x19\n\x15TS_TransportRecording\x10\x02\x12 \n\x1cTS_TransportPlayingHalfSpeed\x10\x03\x12\"\n\x1eTS_TransportRecordingHalfSpeed\x10\x04\x12\x1b\n\x17TS_TransportFastForward\x10\x05\x12\x16\n\x12TS_TransportRewind\x10\x06\x12\x15\n\x11TS_TransportScrub\x10\x07\x12\x17\n\x13TS_TransportShuttle\x10\x08\x12\x16\n\x12TS_TransportPrimed\x10\t\x12\x18\n\x14TS_TransportIsCueing\x10\n\x12\x16\n\x12TS_TransportIsCued\x10\x0b\x12 \n\x1cTS_TransportIsCuedForPreview\x10\x0c\x12\x1a\n\x16TS_TransportIsStopping\x10\r\x12\x1c\n\x18TS_TransportIsPreviewing\x10\x0e*4\n\x0f\x43L_ClipLocation\x12\x10\n\x0c\x43L_ClipsList\x10\x00\x12\x0f\n\x0b\x43L_Timeline\x10\x01*>\n\x0eTimeProperties\x12\r\n\tTP_Marker\x10\x00\x12\x10\n\x0cTP_Selection\x10\x01\x12\x0b\n\x07TP_None\x10\x02*<\n\x17MemoryLocationReference\x12\x0f\n\x0bMLR_BarBeat\x10\x00\x12\x10\n\x0cMLR_Absolute\x10\x01\x32u\n\x04PTSL\x12\x30\n\x0fSendGrpcRequest\x12\r.ptsl.Request\x1a\x0e.ptsl.Response\x12;\n\x18SendGrpcStreamingRequest\x12\r.ptsl.Request\x1a\x0e.ptsl.Response0\x01\x62\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'PTSL_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _COMMANDID._serialized_start=13695
  _COMMANDID._serialized_end=15369
  _TASKSTATUS._serialized_start=15372
  _TASKSTATUS._serialized_end=15539
  _COMMANDERRORTYPE._serialized_start=15542
  _COMMANDERRORTYPE._serialized_end=16547
  _TRACKTYPE._serialized_start=16550
  _TRACKTYPE._serialized_end=16793
  _TRACKATTRIBUTESTATE._serialized_start=16795
  _TRACKATTRIBUTESTATE._serialized_end=16896
  _FILETYPE._serialized_start=16898
  _FILETYPE._serialized_end=16958
  _IOSETTINGS._serialized_start=16961
  _IOSETTINGS._serialized_end=17091
  _IMPORTTYPE._serialized_start=17093
  _IMPORTTYPE._serialized_end=17166
  _AUDIOMEDIAOPTIONS._serialized_start=17169
  _AUDIOMEDIAOPTIONS._serialized_end=17300
  _VIDEOMEDIAOPTIONS._serialized_start=17302
  _VIDEOMEDIAOPTIONS._serialized_end=17404
  _MATCHTRACKOPTIONS._serialized_start=17406
  _MATCHTRACKOPTIONS._serialized_end=17483
  _TIMECODEMAPPINGOPTIONS._serialized_start=17485
  _TIMECODEMAPPINGOPTIONS._serialized_end=17605
  _TRACKOFFSETOPTIONS._serialized_start=17607
  _TRACKOFFSETOPTIONS._serialized_end=17698
  _CONVERSIONQUALITY._serialized_start=17700
  _CONVERSIONQUALITY._serialized_end=17775
  _MAINPLAYLISTOPTIONS._serialized_start=17777
  _MAINPLAYLISTOPTIONS._serialized_end=17892
  _AUDIOOPERATIONS._serialized_start=17894
  _AUDIOOPERATIONS._serialized_end=17971
  _MEDIADESTINATION._serialized_start=17973
  _MEDIADESTINATION._serialized_end=18061
  _MEDIALOCATION._serialized_start=18063
  _MEDIALOCATION._serialized_end=18161
  _TRACKLISTFILTER._serialized_start=18164
  _TRACKLISTFILTER._serialized_end=18490
  _SPOTLOCATIONTYPE._serialized_start=18492
  _SPOTLOCATIONTYPE._serialized_end=18545
  _EXPORTFORMAT._serialized_start=18547
  _EXPORTFORMAT._serialized_end=18628
  _EXPORTFILETYPE._serialized_start=18630
  _EXPORTFILETYPE._serialized_end=18698
  _BITDEPTH._serialized_start=18700
  _BITDEPTH._serialized_end=18762
  _RESOLVEDUPLICATENAMESBY._serialized_start=18764
  _RESOLVEDUPLICATENAMESBY._serialized_end=18834
  _EXPORTASAAFFILETYPE._serialized_start=18836
  _EXPORTASAAFFILETYPE._serialized_end=18915
  _AAFFILEBITDEPTH._serialized_start=18917
  _AAFFILEBITDEPTH._serialized_end=18964
  _COPYOPTION._serialized_start=18966
  _COPYOPTION._serialized_end=19060
  _FILELOCATIONTYPEFILTER._serialized_start=19063
  _FILELOCATIONTYPEFILTER._serialized_end=19295
  _AUTOMATIONDATAOPTIONS._serialized_start=19297
  _AUTOMATIONDATAOPTIONS._serialized_end=19416
  _PASTESPECIALOPTIONS._serialized_start=19418
  _PASTESPECIALOPTIONS._serialized_end=19512
  _TRIPLEBOOL._serialized_start=19514
  _TRIPLEBOOL._serialized_end=19566
  _EM_SOURCETYPE._serialized_start=19568
  _EM_SOURCETYPE._serialized_end=19621
  _COMPRESSIONTYPE._serialized_start=19623
  _COMPRESSIONTYPE._serialized_end=19665
  _SAMPLERATE._serialized_start=19667
  _SAMPLERATE._serialized_end=19778
  _EM_VIDEOEXPORTOPTIONS._serialized_start=19780
  _EM_VIDEOEXPORTOPTIONS._serialized_end=19855
  _EM_FILETYPE._serialized_start=19857
  _EM_FILETYPE._serialized_end=19965
  _EM_FILEDESTINATION._serialized_start=19967
  _EM_FILEDESTINATION._serialized_end=20049
  _EM_DELIVERYFORMAT._serialized_start=20051
  _EM_DELIVERYFORMAT._serialized_end=20136
  _DP_VALUETYPES._serialized_start=20139
  _DP_VALUETYPES._serialized_end=20458
  _DYNAMICPROPERTYTYPE._serialized_start=20460
  _DYNAMICPROPERTYTYPE._serialized_end=20528
  _TRACKLISTTYPE._serialized_start=20530
  _TRACKLISTTYPE._serialized_end=20584
  _FADEHANDLINGTYPE._serialized_start=20586
  _FADEHANDLINGTYPE._serialized_end=20676
  _TEXTASFILEFORMAT._serialized_start=20678
  _TEXTASFILEFORMAT._serialized_end=20720
  _ESI_OUTPUTTYPE._serialized_start=20722
  _ESI_OUTPUTTYPE._serialized_end=20768
  _PM_PLAYBACKMODE._serialized_start=20770
  _PM_PLAYBACKMODE._serialized_end=20840
  _RM_RECORDMODE._serialized_start=20842
  _RM_RECORDMODE._serialized_end=20968
  _SESSIONAUDIOFORMAT._serialized_start=20970
  _SESSIONAUDIOFORMAT._serialized_end=21018
  _SESSIONTIMECODERATE._serialized_start=21021
  _SESSIONTIMECODERATE._serialized_end=21393
  _SESSIONFEETFRAMESRATE._serialized_start=21395
  _SESSIONFEETFRAMESRATE._serialized_end=21469
  _SESSIONRATEPULL._serialized_start=21472
  _SESSIONRATEPULL._serialized_end=21637
  _TS_TRANSPORTSTATE._serialized_start=21640
  _TS_TRANSPORTSTATE._serialized_end=22073
  _CL_CLIPLOCATION._serialized_start=22075
  _CL_CLIPLOCATION._serialized_end=22127
  _TIMEPROPERTIES._serialized_start=22129
  _TIMEPROPERTIES._serialized_end=22191
  _MEMORYLOCATIONREFERENCE._serialized_start=22193
  _MEMORYLOCATIONREFERENCE._serialized_end=22253
  _EMPTYMESSAGE._serialized_start=20
  _EMPTYMESSAGE._serialized_end=34
  _COMMANDERROR._serialized_start=36
  _COMMANDERROR._serialized_end=153
  _REQUESTHEADER._serialized_start=155
  _REQUESTHEADER._serialized_end=258
  _REQUEST._serialized_start=260
  _REQUEST._serialized_end=333
  _RESPONSEHEADER._serialized_start=335
  _RESPONSEHEADER._serialized_end=454
  _RESPONSE._serialized_start=456
  _RESPONSE._serialized_end=561
  _PAGINATION._serialized_start=563
  _PAGINATION._serialized_end=621
  _TRACKATTRIBUTES._serialized_start=624
  _TRACKATTRIBUTES._serialized_end=1058
  _TRACK._serialized_start=1061
  _TRACK._serialized_end=1227
  _GETTASKSTATUSREQUESTBODY._serialized_start=1229
  _GETTASKSTATUSREQUESTBODY._serialized_end=1272
  _GETTASKSTATUSRESPONSEBODY._serialized_start=1274
  _GETTASKSTATUSRESPONSEBODY._serialized_end=1370
  _CREATESESSIONREQUESTBODY._serialized_start=1373
  _CREATESESSIONREQUESTBODY._serialized_end=1778
  _OPENSESSIONREQUESTBODY._serialized_start=1780
  _OPENSESSIONREQUESTBODY._serialized_end=1826
  _TRACKDATATOIMPORT._serialized_start=1828
  _TRACKDATATOIMPORT._serialized_end=1950
  _SESSIONDATAIMPORT._serialized_start=1953
  _SESSIONDATAIMPORT._serialized_end=2151
  _SESSIONDATA._serialized_start=2154
  _SESSIONDATA._serialized_end=2596
  _SPOTLOCATIONDATA._serialized_start=2599
  _SPOTLOCATIONDATA._serialized_end=2740
  _AUDIODATA._serialized_start=2743
  _AUDIODATA._serialized_end=2979
  _VIDEODATA._serialized_start=2981
  _VIDEODATA._serialized_end=2992
  _MIDIDATA._serialized_start=2994
  _MIDIDATA._serialized_end=3004
  _CLIPGROUPSDATA._serialized_start=3006
  _CLIPGROUPSDATA._serialized_end=3022
  _IMPORTREQUESTBODY._serialized_start=3025
  _IMPORTREQUESTBODY._serialized_end=3183
  _IMPORTRESPONSEBODY._serialized_start=3185
  _IMPORTRESPONSEBODY._serialized_end=3299
  _TRACKLISTINVERTIBLEFILTER._serialized_start=3301
  _TRACKLISTINVERTIBLEFILTER._serialized_end=3388
  _GETTRACKLISTREQUESTBODY._serialized_start=3391
  _GETTRACKLISTREQUESTBODY._serialized_end=3529
  _GETTRACKLISTRESPONSEBODY._serialized_start=3531
  _GETTRACKLISTRESPONSEBODY._serialized_end=3623
  _SELECTALLCLIPSONTRACKREQUESTBODY._serialized_start=3625
  _SELECTALLCLIPSONTRACKREQUESTBODY._serialized_end=3679
  _SELECTALLCLIPSONTRACKRESPONSEBODY._serialized_start=3681
  _SELECTALLCLIPSONTRACKRESPONSEBODY._serialized_end=3736
  _EXTENDSELECTIONTOTARGETTRACKSREQUESTBODY._serialized_start=3738
  _EXTENDSELECTIONTOTARGETTRACKSREQUESTBODY._serialized_end=3809
  _EXTENDSELECTIONTOTARGETTRACKSRESPONSEBODY._serialized_start=3811
  _EXTENDSELECTIONTOTARGETTRACKSRESPONSEBODY._serialized_end=3883
  _CREATEFADESBASEDONPRESETREQUESTBODY._serialized_start=3885
  _CREATEFADESBASEDONPRESETREQUESTBODY._serialized_end=3976
  _CREATEFADESBASEDONPRESETRESPONSEBODY._serialized_start=3978
  _CREATEFADESBASEDONPRESETRESPONSEBODY._serialized_end=4042
  _RENAMETARGETTRACKREQUESTBODY._serialized_start=4044
  _RENAMETARGETTRACKREQUESTBODY._serialized_end=4132
  _EXPORTCLIPSASFILESREQUESTBODY._serialized_start=4135
  _EXPORTCLIPSASFILESREQUESTBODY._serialized_end=4389
  _EXPORTSELECTEDTRACKSASAAFOMFREQUESTBODY._serialized_start=4392
  _EXPORTSELECTEDTRACKSASAAFOMFREQUESTBODY._serialized_end=4821
  _REFRESHTARGETAUDIOFILESREQUESTBODY._serialized_start=4823
  _REFRESHTARGETAUDIOFILESREQUESTBODY._serialized_end=4878
  _REFRESHTARGETAUDIOFILESRESPONSEBODY._serialized_start=4880
  _REFRESHTARGETAUDIOFILESRESPONSEBODY._serialized_end=4985
  _FILELOCATIONINFO._serialized_start=4987
  _FILELOCATIONINFO._serialized_end=5024
  _FILELOCATION._serialized_start=5026
  _FILELOCATION._serialized_end=5092
  _GETFILELOCATIONREQUESTBODY._serialized_start=5094
  _GETFILELOCATIONREQUESTBODY._serialized_end=5194
  _GETFILELOCATIONRESPONSEBODY._serialized_start=5196
  _GETFILELOCATIONRESPONSEBODY._serialized_end=5302
  _CLOSESESSIONREQUESTBODY._serialized_start=5304
  _CLOSESESSIONREQUESTBODY._serialized_end=5352
  _SAVESESSIONASREQUESTBODY._serialized_start=5354
  _SAVESESSIONASREQUESTBODY._serialized_end=5428
  _CUTSPECIALREQUESTBODY._serialized_start=5430
  _CUTSPECIALREQUESTBODY._serialized_end=5514
  _COPYSPECIALREQUESTBODY._serialized_start=5516
  _COPYSPECIALREQUESTBODY._serialized_end=5601
  _CLEARSPECIALREQUESTBODY._serialized_start=5603
  _CLEARSPECIALREQUESTBODY._serialized_end=5689
  _PASTESPECIALREQUESTBODY._serialized_start=5691
  _PASTESPECIALREQUESTBODY._serialized_end=5773
  _EM_SOURCEINFO._serialized_start=5775
  _EM_SOURCEINFO._serialized_end=5846
  _EM_AUDIOINFO._serialized_start=5849
  _EM_AUDIOINFO._serialized_end=6128
  _PROPERTYCONTAINER._serialized_start=6130
  _PROPERTYCONTAINER._serialized_end=6223
  _EM_CODECINFO._serialized_start=6225
  _EM_CODECINFO._serialized_end=6307
  _EM_VIDEOINFO._serialized_start=6310
  _EM_VIDEOINFO._serialized_end=6507
  _EM_IMPORTOPTIONS._serialized_start=6510
  _EM_IMPORTOPTIONS._serialized_end=6881
  _EM_LOCATIONINFO._serialized_start=6884
  _EM_LOCATIONINFO._serialized_end=7067
  _EM_DOLBYATMOSINFO._serialized_start=7070
  _EM_DOLBYATMOSINFO._serialized_end=7234
  _EXPORTMIXREQUESTBODY._serialized_start=7237
  _EXPORTMIXREQUESTBODY._serialized_end=7643
  _PROPERTYDESCRIPTOR._serialized_start=7646
  _PROPERTYDESCRIPTOR._serialized_end=7859
  _GETDYNAMICPROPERTIESREQUESTBODY._serialized_start=7861
  _GETDYNAMICPROPERTIESREQUESTBODY._serialized_end=7944
  _GETDYNAMICPROPERTIESGROUP._serialized_start=7946
  _GETDYNAMICPROPERTIESGROUP._serialized_end=8065
  _GETDYNAMICPROPERTIESRESPONSEBODY._serialized_start=8068
  _GETDYNAMICPROPERTIESRESPONSEBODY._serialized_end=8205
  _SPOTREQUESTBODY._serialized_start=8207
  _SPOTREQUESTBODY._serialized_end=8327
  _EXPORTSESSIONINFOASTEXTREQUESTBODY._serialized_start=8330
  _EXPORTSESSIONINFOASTEXTREQUESTBODY._serialized_end=8831
  _EXPORTSESSIONINFOASTEXTRESPONSEBODY._serialized_start=8833
  _EXPORTSESSIONINFOASTEXTRESPONSEBODY._serialized_end=8892
  _SETPLAYBACKMODEREQUESTBODY._serialized_start=8894
  _SETPLAYBACKMODEREQUESTBODY._serialized_end=8968
  _SETPLAYBACKMODERESPONSEBODY._serialized_start=8971
  _SETPLAYBACKMODERESPONSEBODY._serialized_end=9105
  _SETRECORDMODEREQUESTBODY._serialized_start=9107
  _SETRECORDMODEREQUESTBODY._serialized_end=9205
  _SETRECORDMODERESPONSEBODY._serialized_start=9207
  _SETRECORDMODERESPONSEBODY._serialized_end=9331
  _GETSESSIONAUDIOFORMATRESPONSEBODY._serialized_start=9334
  _GETSESSIONAUDIOFORMATRESPONSEBODY._serialized_end=9473
  _GETSESSIONSAMPLERATERESPONSEBODY._serialized_start=9475
  _GETSESSIONSAMPLERATERESPONSEBODY._serialized_end=9548
  _GETSESSIONBITDEPTHRESPONSEBODY._serialized_start=9550
  _GETSESSIONBITDEPTHRESPONSEBODY._serialized_end=9666
  _GETSESSIONINTERLEAVEDSTATERESPONSEBODY._serialized_start=9668
  _GETSESSIONINTERLEAVEDSTATERESPONSEBODY._serialized_end=9760
  _GETSESSIONTIMECODERATERESPONSEBODY._serialized_start=9763
  _GETSESSIONTIMECODERATERESPONSEBODY._serialized_end=9905
  _GETSESSIONFEETFRAMESRATERESPONSEBODY._serialized_start=9908
  _GETSESSIONFEETFRAMESRATERESPONSEBODY._serialized_end=10056
  _GETSESSIONAUDIORATEPULLSETTINGSRESPONSEBODY._serialized_start=10059
  _GETSESSIONAUDIORATEPULLSETTINGSRESPONSEBODY._serialized_end=10202
  _GETSESSIONVIDEORATEPULLSETTINGSRESPONSEBODY._serialized_start=10205
  _GETSESSIONVIDEORATEPULLSETTINGSRESPONSEBODY._serialized_end=10348
  _GETSESSIONNAMERESPONSEBODY._serialized_start=10350
  _GETSESSIONNAMERESPONSEBODY._serialized_end=10400
  _GETSESSIONPATHRESPONSEBODY._serialized_start=10402
  _GETSESSIONPATHRESPONSEBODY._serialized_end=10472
  _GETSESSIONSTARTTIMERESPONSEBODY._serialized_start=10474
  _GETSESSIONSTARTTIMERESPONSEBODY._serialized_end=10535
  _GETSESSIONLENGTHRESPONSEBODY._serialized_start=10537
  _GETSESSIONLENGTHRESPONSEBODY._serialized_end=10591
  _SETSESSIONAUDIOFORMATREQUESTBODY._serialized_start=10593
  _SETSESSIONAUDIOFORMATREQUESTBODY._serialized_end=10675
  _SETSESSIONBITDEPTHREQUESTBODY._serialized_start=10677
  _SETSESSIONBITDEPTHREQUESTBODY._serialized_end=10743
  _SETSESSIONINTERLEAVEDSTATEREQUESTBODY._serialized_start=10745
  _SETSESSIONINTERLEAVEDSTATEREQUESTBODY._serialized_end=10811
  _SETSESSIONTIMECODERATEREQUESTBODY._serialized_start=10813
  _SETSESSIONTIMECODERATEREQUESTBODY._serialized_end=10899
  _SETSESSIONFEETFRAMESRATEREQUESTBODY._serialized_start=10901
  _SETSESSIONFEETFRAMESRATEREQUESTBODY._serialized_end=10993
  _SETSESSIONAUDIORATEPULLSETTINGSREQUESTBODY._serialized_start=10995
  _SETSESSIONAUDIORATEPULLSETTINGSREQUESTBODY._serialized_end=11087
  _SETSESSIONVIDEORATEPULLSETTINGSREQUESTBODY._serialized_start=11089
  _SETSESSIONVIDEORATEPULLSETTINGSREQUESTBODY._serialized_end=11181
  _SETSESSIONSTARTTIMEREQUESTBODY._serialized_start=11184
  _SETSESSIONSTARTTIMEREQUESTBODY._serialized_end=11333
  _SETSESSIONLENGTHREQUESTBODY._serialized_start=11335
  _SETSESSIONLENGTHREQUESTBODY._serialized_end=11388
  _GETPTSLVERSIONRESPONSEBODY._serialized_start=11390
  _GETPTSLVERSIONRESPONSEBODY._serialized_end=11435
  _GETPLAYBACKMODERESPONSEBODY._serialized_start=11438
  _GETPLAYBACKMODERESPONSEBODY._serialized_end=11566
  _GETRECORDMODERESPONSEBODY._serialized_start=11568
  _GETRECORDMODERESPONSEBODY._serialized_end=11689
  _GETTRANSPORTSTATERESPONSEBODY._serialized_start=11692
  _GETTRANSPORTSTATERESPONSEBODY._serialized_end=11825
  _GETTRANSPORTARMEDRESPONSEBODY._serialized_start=11827
  _GETTRANSPORTARMEDRESPONSEBODY._serialized_end=11886
  _CLEARMEMORYLOCATIONREQUESTBODY._serialized_start=11888
  _CLEARMEMORYLOCATIONREQUESTBODY._serialized_end=11943
  _AUTHORIZECONNECTIONREQUESTBODY._serialized_start=11945
  _AUTHORIZECONNECTIONREQUESTBODY._serialized_end=11998
  _AUTHORIZECONNECTIONRESPONSEBODY._serialized_start=12000
  _AUTHORIZECONNECTIONRESPONSEBODY._serialized_end=12093
  _RENAMESELECTEDCLIPREQUESTBODY._serialized_start=12095
  _RENAMESELECTEDCLIPREQUESTBODY._serialized_end=12211
  _RENAMETARGETCLIPREQUESTBODY._serialized_start=12213
  _RENAMETARGETCLIPREQUESTBODY._serialized_end=12300
  _MEMORYLOCATION._serialized_start=12303
  _MEMORYLOCATION._serialized_end=12562
  _MEMORYLOCATIONPROPERTIES._serialized_start=12565
  _MEMORYLOCATIONPROPERTIES._serialized_end=12816
  _EDITMEMORYLOCATIONREQUESTBODY._serialized_start=12819
  _EDITMEMORYLOCATIONREQUESTBODY._serialized_end=13093
  _EDITMEMORYLOCATIONRESPONSEBODY._serialized_start=13095
  _EDITMEMORYLOCATIONRESPONSEBODY._serialized_end=13127
  _CREATEMEMORYLOCATIONREQUESTBODY._serialized_start=13130
  _CREATEMEMORYLOCATIONREQUESTBODY._serialized_end=13406
  _CREATEMEMORYLOCATIONRESPONSEBODY._serialized_start=13408
  _CREATEMEMORYLOCATIONRESPONSEBODY._serialized_end=13442
  _GETMEMORYLOCATIONSREQUESTBODY._serialized_start=13444
  _GETMEMORYLOCATIONSREQUESTBODY._serialized_end=13475
  _GETMEMORYLOCATIONSRESPONSEBODY._serialized_start=13477
  _GETMEMORYLOCATIONSRESPONSEBODY._serialized_end=13557
  _REGISTERCONNECTIONREQUESTBODY._serialized_start=13559
  _REGISTERCONNECTIONREQUESTBODY._serialized_end=13638
  _REGISTERCONNECTIONRESPONSEBODY._serialized_start=13640
  _REGISTERCONNECTIONRESPONSEBODY._serialized_end=13692
  _PTSL._serialized_start=22255
  _PTSL._serialized_end=22372
# @@protoc_insertion_point(module_scope)



import json

from ptsl import PTSL_pb2 as pt

from ptsl.ops import Operation


class GetPlaybackMode(Operation):
    pass

    def json_cleanup(self, in_json: str) -> str:

        def xform(x):
            return pt.PM_PlaybackMode.Value(x)

        decoder = json.JSONDecoder()
        decoded = decoder.decode(in_json)
        if isinstance(['possible_settings'][0], str):
            decoded['current_settings'] = [
                xform(x) for x in decoded['current_settings']]
            decoded['possible_settings'] = [
                xform(x) for x in decoded['possible_settings']]
        encoder = json.JSONEncoder()
        encoded = encoder.encode(decoded)

        return encoded



"""
ptsl Engine - Engine class and context manager
"""

from __future__ import annotations
from typing import Optional, Tuple, List

from contextlib import contextmanager

import ptsl
from ptsl import ops
from ptsl.builders.create_session_builder import \
    CreateSessionBuilder, CreateSessionFromTemplateBuilder, \
    CreateSessionFromAAFBuilder
from ptsl.builders.export_text_builder import \
    ExportSessionTextBuilder
from ptsl.builders.import_builder import \
    ImportSessionDataBuilder

import ptsl.PTSL_pb2 as pt
from ptsl.PTSL_pb2 import SessionAudioFormat, BitDepth,  FileLocation, \
    EM_FileType, EM_SourceInfo, EM_AudioInfo, EM_VideoInfo, \
    EM_LocationInfo, EM_DolbyAtmosInfo, TripleBool, SessionTimeCodeRate, \
    SessionFeetFramesRate, SessionRatePull, Track, \
    PM_PlaybackMode, RM_RecordMode, AutomationDataOptions, \
    PasteSpecialOptions, TrackOffsetOptions, TrackListInvertibleFilter, \
    ExportFileType, ResolveDuplicateNamesBy, ExportFormat, \
    MemoryLocationReference, MemoryLocationProperties, \
    TimeProperties, CL_ClipLocation


@contextmanager
def open_engine(*args, **kwargs):
    """
    Open a ptsl engine. Engine will close with the context.
    """
    engine = Engine(*args, **kwargs)

    try:
        yield engine
    finally:
        engine.close()


class Engine:
    """
    A callable interface for PTSL.

    The Engine exposes PTSL commands as methods, translating call
    arguments into corresponding requests, and then translating
    responses into return objects. So, instead of creating a request,
    dispatching a command to the client with the request object,
    receving a response (or error) and processing it, you can simply
    call a method on the Engine with parameters, and that method returns
    a value.

    One of the goals of the engine class is to hide as many redundant
    value classes from the PTSL protocol as possible, so where the PTSL
    client may return an enumeration value for the session sample rate,
    the Engine returns a simple integer. Entity types, like :class:`Track`
    or :class:`MemoryLocation` objects are retained.

    The `Engine` initializes a new :class:`Client` object by passing its
    initialization parameters to :meth:`~ptsl.Client.__init__`
    """

    client: ptsl.Client

    def __init__(self,
                 company_name: Optional[str] = None,
                 application_name: Optional[str] = None,
                 certificate_path: Optional[str] = None,
                 address='localhost:31416'):

        self.client = ptsl.Client(certificate_path=certificate_path,
                                  company_name=company_name,
                                  application_name=application_name,
                                  address=address)

    def close(self):
        """
        Close the engine.
        """
        self.client.close()

    def ptsl_version(self) -> int:
        """
        Requests the current PTSL version running on the server and returns the
        reponse value.

        :returns: The server's PTSL version number.
        :rtype: int
        """
        op = ops.GetPTSLVersion()
        self.client.run(op)
        assert isinstance(
            op.response, pt.GetPTSLVersionResponseBody), \
            f"Expected response body, encountered {op.response}"

        return op.response.version

    def host_ready_check(self):
        """
        Runs the `HostReadyCheck` message on the host, any error
        is returned as an exception.

        .. note:: This method will succeed even if the client
            connection is not registered.
        """
        self.client.run(ops.HostReadyCheck())

    def create_session(self,
                       name: str,
                       path: str) -> CreateSessionBuilder:
        """
        Create a new Pro Tools session.

        :param str name: Session Name
        :param str path: Path to the new session
        """
        return CreateSessionBuilder(engine=self, name=name, path=path)

    def create_session_from_template(
            self,
            template_group: str,
            template_name: str,
            name: str,
            path: str) -> CreateSessionFromTemplateBuilder:
        """
        Create a new session with an installed template.

        :param str template_group: Name of the template group
        :param str template_name: Name of the template to use
        :param str name: Name of the new session
        :param str path: Path for the new session
        """
        return CreateSessionFromTemplateBuilder(self,
                                                template_name,
                                                template_group,
                                                name,
                                                path)

    def create_session_from_aaf(
            self,
            name: str,
            path: str,
            aaf_path: str) -> CreateSessionFromAAFBuilder:
        """
        Create a session from an AAF.

        :param str name: Name of the new session
        :param str path: Path for the new session
        :param str aaf_path: Path to the AAF file to convert
        """
        return CreateSessionFromAAFBuilder(self,
                                           aaf_path=aaf_path,
                                           name=name,
                                           path=path)

    def open_session(self, path: str):
        """
        Open a session.
        """
        op = ops.OpenSession(session_path=path)
        self.client.run(op)

    def close_session(self, save_on_close: bool):
        """
        Close the currently-open session.
        """
        op = ops.CloseSession(save_on_close=save_on_close)
        self.client.run(op)

    def save_session(self):
        """
        Save the currently-open session.
        """
        op = ops.SaveSession()
        self.client.run(op)

    def save_session_as(self, path: str, name: str):
        """
        Save the currently-open session as a new name to a different path.

        :param str path: Path to the new session
        :param str name: New name for the session
        """
        op = ops.SaveSessionAs(session_name=name, session_location=path)
        self.client.run(op)

    def export_session_as_text(self) -> ExportSessionTextBuilder:
        """
        Export the open session as text.
        """
        return ExportSessionTextBuilder(self)

    def import_data(self,
                    session_path: str
                    ) -> ImportSessionDataBuilder:
        """
        Import session data into the currently-open session.
        """
        return ImportSessionDataBuilder(self, session_path)

    def select_all_clips_on_track(self, track_name: str):
        """
        Select all clips on track.

        :param str track_name: Name of the track to select all clips on.
        """
        op = ops.SelectAllClipsOnTrack(track_name=track_name)
        self.client.run(op)

    def extend_selection_to_target_tracks(self, tracks: List[str]):
        """
        Extend selection to target tracks.

        :param List[str] tracks: A list of track names to extend
            the selection to.
        """
        op = ops.ExtendSelectionToTargetTracks(tracks_to_extend_to=tracks)
        self.client.run(op)

    def trim_to_selection(self):
        """
        Trim selected clips to the edit selection range.
        """
        op = ops.TrimToSelection()
        self.client.run(op)

    def create_batch_fades(self, preset_name: str, adjust_bounds: bool):
        """
        Create batch fades for the timeline selection based on a preset.

        :param str preset_name: Name of the batch fades preset to use.
        :param bool adjust_bounds: Auto-adjust clip boundaries to accomodate
            fades.
        """
        rq = pt.CreateFadesBasedOnPresetRequestBody()
        rq.fade_preset_name = preset_name
        rq.auto_adjust_bounds = adjust_bounds
        op = ops.CreateFadesBasedOnPreset()
        op.request = rq
        self.client.run(op)

    def rename_target_track(self, old_name: str, new_name: str):
        """
        Renames a track in the currently-open session.

        :param str old_name: The name of the track to rename.
        :param str new_name: The new name to give the track.
        """
        op = ops.RenameTargetTrack(current_name=old_name,
                                   new_name=new_name)
        self.client.run(op)

    def rename_selected_clip(
            self,
            new_name: str,
            rename_file: bool = True,
            clip_location: 'CL_ClipLocation' = pt.CL_Timeline):
        """
        Renames a clip in the current session.

        :param str new_name: New name for the clip
        :param bool rename_file: If true, file will be renamed as well
        :param CL_ClipLocation clip_location: Clip selection location,
            defaults to :attr:`~ptsl.PTSL_pb2.CL_ClipLocation.CL_Timeline`
        """
        op = ops.RenameSelectedClip(clip_location=clip_location,
                                    new_name=new_name,
                                    rename_file=rename_file)

        self.client.run(op)

    def rename_target_clip(self, clip_name: str,
                           new_name: str,
                           rename_file: bool = True):
        """
        Renames a named clip in the current session.
        """
        op = ops.RenameTargetClip(clip_name=clip_name,
                                  new_name=new_name,
                                  rename_file=rename_file)

        self.client.run(op)

    def toggle_play_state(self):
        """
        Toggle the play state.
        """
        self.client.run(ops.TogglePlayState())

    def toggle_record_enable(self):
        """
        Toggle record enable.
        """
        self.client.run(ops.ToggleRecordEnable())

    def play_half_speed(self):
        """
        Play at half speed.
        """
        self.client.run(ops.PlayHalfSpeed())

    def record_half_speed(self):
        """
        Record at half speed.
        """
        self.client.run(ops.RecordHalfSpeed())

    def edit_memory_location(self, location_number: int,
                             name: str,
                             start_time: str, end_time: str,
                             time_properties: 'TimeProperties',
                             reference: 'MemoryLocationReference',
                             general_properties: 'MemoryLocationProperties',
                             comments: str):
        """
        Edit a memory location.

        :param int location_number: Location number to edit (if location
            does not exist, this will create a new location in 2023.6)
        :param str name: Location name
        :param str start_time: Start time
        :param str end_time: End time
        :param TimeProperties time_properties: Time properties, either this
            is a range or a marker
        :param MemoryLocationReference reference: Reference
        :param MemoryLocationProperties general_properties: Location properties
        :param str comments: Comment field
        """
        op = ops.EditMemoryLocation(
            number=location_number,
            name=name,
            start_time=start_time,
            end_time=end_time,
            time_properties=time_properties,
            reference=reference,
            general_properties=general_properties,
            comments=comments
        )

        self.client.run(op)

    def get_memory_locations(self) -> List[pt.MemoryLocation]:
        """
        Get a list of all memory locations in currently-open session.
        """
        op = ops.GetMemoryLocations()
        self.client.run(op)
        return op.response.memory_locations

    def consolidate_clip(self):
        """
        Consolidate time selection.
        """
        op = ops.ConsolidateClip()
        self.client.run(op)

    def export_clips_as_files(
            self,
            path: str,
            ftype: 'ExportFileType', bit_depth: 'BitDepth',
            ex_format: Optional['ExportFormat'] = None,
            enforce_avid_compatibility: bool = False,
            resolve_duplicates: Optional['ResolveDuplicateNamesBy'] = None
    ):
        """
        Export clips as files.

        :param path: Export directory path. (A MacOS path to a folder, must
            end with a colon ":".)
        :param ftype: File type, WAV/AIFF/etc.
        :param bit_depth: Bit Depth
        :param ex_format: Export file format, mono/multiple mono/interleaved
        :param enforce_avid_compatibilty: Enforce Avid compatibility
        :param resolve_duplicates: Duplicate name resolution method
        :raises: :class:`~ptls.errors.CommandError` A :attr:`PT_UnknownError`
            if the ``path`` argument does not end with a colon.
        """
        rq = pt.ExportClipsAsFilesRequestBody()
        rq.bit_depth = bit_depth
        rq.enforce_avid_compatibility = enforce_avid_compatibility
        rq.file_path = path
        rq.file_type = ftype

        if resolve_duplicates is not None:
            rq.duplicate_names = resolve_duplicates

        if ex_format is not None:
            rq.format = ex_format

        op = ops.ExportClipsAsFiles()
        op.request = rq
        self.client.run(op)

    def get_file_location(
            self,
            filters=None) -> List['FileLocation']:
        """
        Get a list of file locations meeting a set of criteria.

        :param filters: a List of
            :py:class:`~ptsl.PTSL_pb2.FileLocationTypeFilter`
            If none, defaults
            to [:attr:`~ptsl.PTSL_pb2.FileLocationTypeFilter.All_Files`]
        :returns: a List of :py:class:`~ptsl.PTSL_pb2.FileLocation`
        """
        if filters is None:
            filters = [pt.All_Files]
        op = ops.GetFileLocation(
            page_limit=10000,
            file_filters=filters)
        self.client.run(op)

        return op.response.file_locations

    def export_mix(
            self,
            base_name: str,
            file_type: 'EM_FileType',
            sources: List['EM_SourceInfo'],
            audio_info: 'EM_AudioInfo',
            video_info: 'EM_VideoInfo',
            location_info: 'EM_LocationInfo',
            dolby_atmos_info: 'EM_DolbyAtmosInfo',
            offline_bounce: 'TripleBool'
    ):
        """
        Export mixes/"Bounce to Disk" busses in the currently-open session.

        .. note:: This method runs synchronously and will not return until the
                bounce has completed.

        :param EM_FileType file_type: Export file type
        :param List[EM_SourceInfo] sources: Busses to bounce
        :param EM_AudioInfo audio_info: Audio options
        :param EM_VideoInfo video_info: Video options
        :param EM_LocationInfo location_info: Output folder settings
        :param EM_DolbyAtmosInfo dolby_atmos_info: Dolby Atmos output settings
        :param bool offline_bounce: Bounce offline option
        """
        op = ops.ExportMix(
            file_name=base_name,
            file_type=file_type,
            mix_source_list=sources,
            audio_info=audio_info,
            video_info=video_info,
            location_info=location_info,
            dolby_atmos_info=dolby_atmos_info,
            offline_bounce=offline_bounce
        )
        self.client.run(op)

    def session_name(self) -> str:
        """
        Name of the current open session.
        """
        op = ops.GetSessionName()
        self.client.run(op)
        return op.response.session_name

    def session_path(self) -> str:
        """
        Path to the current open session.
        """
        op = ops.GetSessionPath()
        self.client.run(op)
        return op.response.session_path.path

    def session_sample_rate(self) -> Optional[int]:
        """
        Open session sample rate.
        """
        op = ops.GetSessionSampleRate()
        self.client.run(op)
        map_dict = {
            pt.SR_192000: 192000,
            pt.SR_176400: 176400,
            pt.SR_96000: 96000,
            pt.SR_88200: 88200,
            pt.SR_48000: 48000,
            pt.SR_44100: 44100
        }
        return map_dict.get(op.response.sample_rate, None)

    def session_audio_format(self) -> 'SessionAudioFormat':
        """
        Open session audio format.
        """
        op = ops.GetSessionAudioFormat()
        self.client.run(op)
        return op.response.current_setting

    def session_bit_depth(self) -> 'BitDepth':
        op = ops.GetSessionBitDepth()
        self.client.run(op)
        return op.response.current_setting

    def session_interleaved_state(self) -> bool:
        """
        Session audio file interleaved state.
        """
        op = ops.GetSessionInterleavedState()
        self.client.run(op)
        return op.response.current_setting

    def session_timecode_rate(self) -> 'SessionTimeCodeRate':
        """
        Session timecode rate.
        """
        op = ops.GetSessionTimeCodeRate()
        self.client.run(op)
        return op.response.current_setting

    def session_start_time(self) -> str:
        """
        Session start time.
        """
        op = ops.GetSessionStartTime()
        self.client.run(op)
        return op.response.session_start_time

    def session_length(self) -> str:
        """
        Session length.

        :returns: Session length, as a string in the current
            time code format.
        """
        op = ops.GetSessionLength()
        self.client.run(op)
        return op.response.session_length

    def session_feet_frames_rate(self) -> 'SessionFeetFramesRate':
        """
        Session feet-frames rate.
        """
        op = ops.GetSessionFeetFramesRate()
        self.client.run(op)
        return op.response.current_setting

    def session_audio_rate_pull(self) -> 'SessionRatePull':
        """
        Audio pull setting of the currently-open session.
        """
        op = ops.GetSessionAudioRatePullSettings()
        self.client.run(op)
        return op.response.current_setting

    def session_video_rate_pull(self) -> 'SessionRatePull':
        """
        Video pull setting of the currently-open session.
        """
        op = ops.GetSessionVideoRatePullSettings()
        self.client.run(op)
        return op.response.current_setting

    def transport_state(self) -> str:
        """
        Current transport state.
        """
        op = ops.GetTransportState()
        self.client.run(op)
        return pt.TS_TransportState.Name(op.response.current_setting)

    def transport_armed(self) -> bool:
        """
        Transport record-arm state.
        """
        op = ops.GetTransportArmed()
        self.client.run(op)
        return op.response.is_transport_armed

    def playback_modes(self) -> Tuple[bool, bool, bool]:
        """
        Transport's current set of playback modes.

        :returns: A Tuple of (`is_normal`, `is_loop`, `is_dynamic_transport`)
        """
        op = ops.GetPlaybackMode()
        self.client.run(op)
        return (pt.PM_Normal in op.response.current_settings,
                pt.PM_Loop in op.response.current_settings,
                pt.PM_DynamicTransport in op.response.current_settings)

    def record_mode(self) -> 'RM_RecordMode':
        """
        Transport's current record mode.
        """
        op = ops.GetRecordMode()
        self.client.run(op)
        return op.response.current_setting

    def track_list(
            self,
            filters: List['TrackListInvertibleFilter'] = []
    ) -> List['Track']:
        """
        Get a list of the tracks in the current session.

        :param filters: Track list filters. Defaults to
            [:attr:`ptsl.PTSL_pb2.TrackListFilter.All`]
        """
        if len(filters) == 0:
            filters = [pt.TrackListInvertibleFilter(filter=pt.All,
                                                    is_inverted=False)]

        op = ops.GetTrackList(
            page_limit=1000,
            track_filter_list=filters
        )

        self.client.run(op)

        return op.track_list

    def set_playback_mode(self, new_mode: 'PM_PlaybackMode'):
        """
        Set the playback mode.
        """
        op = ops.SetPlaybackMode(playback_mode=new_mode)
        self.client.run(op)

    def set_record_mode(
            self,
            new_mode: 'RM_RecordMode',
            record_arm_transport: bool):
        """
        Set the record mode.
        """
        op = ops.SetRecordMode(
            record_mode=new_mode,
            record_arm_transport=record_arm_transport
        )
        self.client.run(op)

    def set_session_bit_depth(self, new_bit_depth: 'BitDepth'):
        """
        Set session bit depth.
        """
        op = ops.SetSessionBitDepth(bit_depth=new_bit_depth)
        self.client.run(op)

    def set_session_audio_format(self, new_audio_format: 'SessionAudioFormat'):
        """
        Set session audio format.
        """
        op = ops.SetSessionAudioFormat(audio_format=new_audio_format)
        self.client.run(op)

    def set_session_start_time(
            self,
            new_start: str,
            track_offset_opts: 'TrackOffsetOptions',
            maintain_relative: bool
    ):
        """
        Set session start time.

        :param new_start: New timecode start time
        :param track_offset_opts: The time format of ``new_start``.
        :param maintain_relative: If `True`, clips will retain their time
            position relative to the beginning of the session.
        """
        op = ops.SetSessionStartTime(
            session_start_time=new_start,
            track_offset_opts=track_offset_opts,
            maintain_relative_position=maintain_relative
        )

        self.client.run(op)

    def set_session_length(self, new_length: str):
        """
        Set the session length as a timecode value string.

        .. important:: The ``new_length`` value must be greater
            than "06:00:00:00", the PTSL server will reject the
            change and return an error otherwise.
        """
        op = ops.SetSessionLength(session_length=new_length)
        self.client.run(op)

    def set_session_interleaved_state(self, new_state: bool):
        """
        Set session interleaved state.
        """
        op = ops.SetSessionInterleavedState(interleaved_state=new_state)
        self.client.run(op)

    def set_session_time_code_rate(self, tc_rate: 'SessionTimeCodeRate'):
        """
        Set session timecode rate.
        """
        op = ops.SetSessionTimeCodeRate(time_code_rate=tc_rate)
        self.client.run(op)

    def set_session_feet_frames_rate(self, ff_rate: 'SessionFeetFramesRate'):
        """
        Set session feet+frames rate.
        """
        op = ops.SetSessionFeetFramesRate(feet_frames_rate=ff_rate)
        self.client.run(op)

    def set_session_audio_rate_pull(self, pull_rate: 'SessionRatePull'):
        """
        Set session audio rate pull.
        """
        op = ops.SetSessionAudioRatePullSettings(audio_rate_pull=pull_rate)
        self.client.run(op)

    def set_session_video_rate_pull(self, pull_rate: 'SessionRatePull'):
        """
        Set session video rate pull.
        """
        op = ops.SetSessionVideoRatePullSettings(video_rate_pull=pull_rate)
        self.client.run(op)

    def cut(self, special: Optional['AutomationDataOptions'] = None):
        """
        Execute an Edit > Cut.
        """
        if special is not None:
            op = ops.CutSpecial(automation_data_option=special)
        else:
            op = ops.Cut()

        self.client.run(op)

    def copy(self, special: Optional['AutomationDataOptions'] = None):
        """
        Execute an Edit > Copy.
        """
        if special is not None:
            op = ops.CopySpecial(automation_data_option=special)
        else:
            op = ops.Copy()

        self.client.run(op)

    def paste(self, special: Optional['PasteSpecialOptions'] = None):
        """
        Execute an Edit > Paste.
        """
        if special is not None:
            op = ops.PasteSpecial(paste_special_option=special)
        else:
            op = ops.Paste()

        self.client.run(op)

    def clear(self, special: Optional['AutomationDataOptions'] = None):
        """
        Execute an Edit > Clear.
        """
        if special is not None:
            op = ops.ClearSpecial(automation_data_option=special)
        else:
            op = ops.Clear()

        self.client.run(op)

    def refresh_target_audio_files(self, files: List[str]):
        """
        Refresh target audio files.

        :param files: A list of files to refresh.
        """
        op = ops.RefreshAllModifiedAudioFiles(file_list=files)
        self.client.run(op)

    def refresh_all_modified_audio_flles(self):
        """
        Refreshes all modified audio files.
        """
        self.client.run(ops.RefreshAllModifiedAudioFiles())



from typing import TypeVar, Generic

from ptsl import PTSL_pb2 as pt

Q = TypeVar("Q")
R = TypeVar("R")


class Operation(Generic[Q, R]):
    """
    An operation composes a `CommandId` with its request and response type.
    The `Operation` abstract base class will infer the `CommandId` Request
    and Response type from the class name, and will look up the command ID
    code number based on its class name. These can be overridden in subclasses
    for special cases but most of the time simply naming a class after its
    CommandId enum name and deriving it from `Operation` will be sufficient.

    The client runs `Operation`s with the Client.run() method.
    """
    request: Q
    response: R

    @classmethod
    def request_body(cls) -> Q:
        return getattr(pt, cls.__name__ + "RequestBody", None)  # type: ignore

    @classmethod
    def response_body(cls) -> R:
        return getattr(pt, cls.__name__ + "ResponseBody", None)  # type: ignore

    @classmethod
    def command_id(cls):
        return getattr(pt, cls.__name__, -1)

    def __init__(self, *args, **kwargs) -> None:
        rq = self.__class__.request_body()
        if rq is not None:
            self.request = rq(**kwargs)  # type: ignore
        else:
            self.request = None  # type: ignore

        self.response = None  # type: ignore
        self.status = None
        self.task_id = ""

    def json_messup(self, in_json: str) -> str:
        """
        A shim that can be overriden by subclasses to adapt the json the
        Protobuf machinery creates into what Pro Tools expects. Necessary
        if the server's request parser is buggy/not doing what the .proto
        file says it should

        :param in_json: json from protobuf's `MessageToJson` method
        :returns: json to be used in the operation's request_body
        """
        return in_json

    def json_cleanup(self, in_json: str) -> str:
        """
        A shim that can be overriden by subclases to adapt the json Pro
        Tools generates in response to what Protobuf expects. Necessary if
        the Pro Tools response JSON is buggy/not doing what the .proto file
        says it should. (Happens sometimes!)

        :param in_json: json from the ResponseBody Pro Tools sent.
        :returns: the json that will be passed to Protobuf's
        `json_format.Parse` method.
        """
        return in_json

    def on_empty_response_body(self):
        """
        The client calls this when the server reponds.
        """
        self.response = None  # type: ignore

    def on_response_body(self, response_body: R):
        """
        The client calls this when the server responds with a JSON
        `pt.ResponseBody`.
        """
        self.response = response_body



"""
Native Python PTSL (Pro Tools Scripting Library) RPC interface

This package (c) 2023 Jamie Hardt. All Rights Reserved.
Read the LICENSE file distributed with this package to learn about
your rights to use, modify and redistribute this software package.

This project is a work of Pro Tools enthusiasts and users and is not
affiliated with Avid.

For more information on this package see the github repository at
https://github.com/iluvcapra/py-ptsl/

Note on Versions
----------------

The `__version__` property of this module uses semantic versioning of
a particular form.

- The first element of the version triple is the PTSL version as
  supported by the client * 100, plus the major version of this module.
- The second element increments with feature enhancements. Support for
  new versions of Pro Tools will always increment this value.
- The third element increments with bug fixes or modifcations to docs,
  build system, etc.


"""

from .client import Client
from .engine import Engine, open_engine
from .errors import CommandError

__version__ = '101.0.0'



from typing import List
import json

from ptsl import PTSL_pb2 as pt

from ptsl.ops import Operation


class GetTrackList(Operation):

    def __init__(self, **kwargs) -> None:
        self.track_list: List[pt.Track] = []
        super().__init__(**kwargs)

    def json_cleanup(self, in_json: str) -> str:

        def empty_dict_to_empty_list(dct):
            if 'track_list' in dct and dct['track_list'] == {}:
                dct['track_list'] = []

            return dct

        decoded = json.loads(in_json, object_hook=empty_dict_to_empty_list)
        return json.dumps(decoded)

    def on_empty_response_body(self):
        self.track_list = []

    def on_response_body(self, body: pt.GetTrackListResponseBody):
        self.track_list.extend(body.track_list)



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

from . import PTSL_pb2 as PTSL__pb2


class PTSLStub(object):
    """*
    Service for handling different types of ProTools commands using PTSL Client.
    """

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.SendGrpcRequest = channel.unary_unary(
                '/ptsl.PTSL/SendGrpcRequest',
                request_serializer=PTSL__pb2.Request.SerializeToString,
                response_deserializer=PTSL__pb2.Response.FromString,
                )
        self.SendGrpcStreamingRequest = channel.unary_stream(
                '/ptsl.PTSL/SendGrpcStreamingRequest',
                request_serializer=PTSL__pb2.Request.SerializeToString,
                response_deserializer=PTSL__pb2.Response.FromString,
                )


class PTSLServicer(object):
    """*
    Service for handling different types of ProTools commands using PTSL Client.
    """

    def SendGrpcRequest(self, request, context):
        """*
        Send generic gRPC request and receive generic response.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def SendGrpcStreamingRequest(self, request, context):
        """*
        Send generic gRPC request and receive generic streaming responses.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_PTSLServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'SendGrpcRequest': grpc.unary_unary_rpc_method_handler(
                    servicer.SendGrpcRequest,
                    request_deserializer=PTSL__pb2.Request.FromString,
                    response_serializer=PTSL__pb2.Response.SerializeToString,
            ),
            'SendGrpcStreamingRequest': grpc.unary_stream_rpc_method_handler(
                    servicer.SendGrpcStreamingRequest,
                    request_deserializer=PTSL__pb2.Request.FromString,
                    response_serializer=PTSL__pb2.Response.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'ptsl.PTSL', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class PTSL(object):
    """*
    Service for handling different types of ProTools commands using PTSL Client.
    """

    @staticmethod
    def SendGrpcRequest(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/ptsl.PTSL/SendGrpcRequest',
            PTSL__pb2.Request.SerializeToString,
            PTSL__pb2.Response.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def SendGrpcStreamingRequest(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_stream(request, target, '/ptsl.PTSL/SendGrpcStreamingRequest',
            PTSL__pb2.Request.SerializeToString,
            PTSL__pb2.Response.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)



from ptsl.ops import Operation


# There is a documented bug with this, it won't return a string
# if you select ESI_string as the output_type
#
# https://duc.avid.com/showthread.php?t=422971
class ExportSessionInfoAsText(Operation):

    @classmethod
    def response_body(cls):
        return None



"""
ptsl Client - Client class and client context manager.
"""

import io
from contextlib import contextmanager
import json
import sys
import time

from typing import Optional

import grpc
from google.protobuf import json_format

from ptsl import PTSL_pb2_grpc
from ptsl import PTSL_pb2 as pt
from ptsl.errors import CommandError
from ptsl.ops import Operation

PTSL_VERSION = 1


@contextmanager
def open_client(*args, **kwargs):
    client = Client(*args, **kwargs)

    try:
        yield client
    finally:
        client.close()


class Auditor:
    """
    The Auditor is used by the client for reporting-out the status of requests
    as they are run.
    """

    def __init__(self, enabled: bool) -> None:
        self.output_stream = sys.stderr
        self.command_sn = 1
        self.enabled = enabled

    def emit(self, message):
        if self.enabled:
            time_str = time.strftime("[%Y-%m-%d %H:%M:%S]")
            print("%04i%s %s" % (self.command_sn, time_str, message),
                  file=self.output_stream)

    def run_called(self, command: pt.CommandId):
        self.emit(f"Started Command {pt.CommandId.Name(command)} " +
                  "({command})")

    def request_json_before_cleanup(self, json_data):
        self.emit(f"Created JSON for request message: {json_data}")

    def request_json_after_cleanup(self, json_data):
        self.emit(f"Re-formatted JSON for request message: {json_data}")

    def response_json_before_cleanup(self, json_data):
        self.emit(f"Received JSON response body: {json_data}")

    def response_json_after_cleanup(self, json_data):
        self.emit(f"Re-formatted JSON response body: {json_data}")

    def response_was_empty(self):
        self.emit("Received empty JSON response")

    def run_returning(self):
        self.emit("Finished Command")
        self.emit(f"{('*' * 60)}\n\n")
        self.command_sn += 1


class Client:
    """
    Medium-level PTSL interface.

    The Client class:
        - maintains the grpc stub and channel
        - holds PTSL server session data
        - manages the connection's registration
        - runs `Operation`s on the grpc channel
        - packages error responses as exceptions to be thrown.
    """

    channel: grpc.Channel
    raw_client: PTSL_pb2_grpc.PTSLStub
    session_id: str
    auditor: Auditor
    is_open: bool

    def __init__(self,
                 company_name: Optional[str] = None,
                 application_name: Optional[str] = None,
                 certificate_path: Optional[str] = None,
                 address: str = 'localhost:31416') -> None:
        """
        Creates a new client.

        ..note:: If `certificate_path` is given, the legacy AuthorizeConnection
            method will be used for setting up the connection session. If it is
            `None`, then `company_name` and `application_name` will be used
            with the RegisterConnection method (available since Pro Tools
            2023.3).
        """

        self.channel = grpc.insecure_channel(address)
        self.raw_client = PTSL_pb2_grpc.PTSLStub(self.channel)
        self.session_id = ""
        self.auditor = Auditor(enabled=False)

        try:
            self._primitive_check_if_ready()

            if certificate_path is not None:
                self._primitive_authorize_connection(certificate_path)

            elif company_name is not None and application_name is not None:
                self._primitive_register_connection(
                    company_name, application_name)

            else:
                raise AssertionError(
                    "company_name and application_name parameters were " +
                    "not given")

            self.is_open = True

        except grpc.RpcError as grpc_error:
            self.close()
            if getattr(grpc_error, 'code')() == grpc.StatusCode.UNAVAILABLE:
                print("gRPC endpoint was unavailable, Pro Tools " +
                      "may not be running.", file=sys.stderr)

            raise grpc_error

    def run(self, operation: Operation) -> None:
        """
        Run an operation on the client.

        :raises: `CommandError` if the server returns an error
        """

        self.auditor.run_called(operation.command_id())

        # convert the request body into JSON
        request_body_json = self._prepare_operation_request_json(operation)
        response = self._send_sync_request(operation.command_id(),
                                           request_body_json)
        operation.status = response.header.status

        if response.header.status == pt.Failed:
            cleaned_response_error_json = self._response_error_json_cleanup(
                response.response_error_json)
            command_error = json_format.Parse(cleaned_response_error_json,
                                              pt.CommandError())
            raise CommandError(command_error)

        elif response.header.status == pt.Completed:
            self._handle_completed_response(operation, response)
        else:
            # FIXME: dump out for now, will be on the lookout for when
            # this happens
            assert False, \
                f"Unexpected response code {response.header.status} " + \
                "({pt.TaskStatus.Name(response.header.status)})"

        self.auditor.run_returning()

    def _prepare_operation_request_json(self, operation):
        """
        Convert the request body into a JSON string (or an empty string if
        there is no request body.
        """
        if operation.request is None:
            request_body_json = ""
        else:
            request_body_json = \
                json_format.MessageToJson(operation.request,
                                          including_default_value_fields=True,
                                          preserving_proto_field_name=True)

        self.auditor.request_json_before_cleanup(request_body_json)
        request_body_json = operation.json_messup(request_body_json)
        self.auditor.request_json_after_cleanup(request_body_json)
        return request_body_json

    def _response_error_json_cleanup(self, json_in: str) -> str:
        """
        This is a shim that will take a `command_error_type` value
        from the response error json and convert it into a PT_UnknownError
        if the `command_error_type` value is mis-formatted by the server
        (for instance, if the server returns a symbold name or numeric string
        value, as it sometimes has done in the past. (See `errata`).
        """
        error_dict = json.loads(json_in)
        old_val = error_dict['command_error_type']
        if isinstance(old_val, str):
            if old_val.isdigit():
                error_dict['command_error_type'] = int(old_val)
            elif old_val in pt.CommandErrorType.keys():
                error_dict['command_error_type'] = \
                    pt.CommandErrorType.Value(old_val)
            else:
                error_dict['command_error_type'] = pt.PT_UnknownError

        return json.dumps(error_dict)

    def _handle_completed_response(self, operation, response):
        """
        Accept the response message from the server, parse
        the response body JSON if present, and hand the results
        to the operation.

        If the operation provides a cleanup function, this is run on
        the response body JSON prior to parsing.
        """
        p = operation.__class__.response_body()
        if len(response.response_body_json) > 0 and p is not None:
            self.auditor.response_json_before_cleanup(
                response.response_body_json)

            clean_json = operation.json_cleanup(response.response_body_json)
            self.auditor.response_json_after_cleanup(clean_json)
            resp_body = json_format.Parse(clean_json, p(),
                                          ignore_unknown_fields=True)
            operation.on_response_body(resp_body)
        else:
            operation.on_empty_response_body()
            self.auditor.response_was_empty()

    def _send_sync_request(self, command_id,
                           request_body_json, task_id="") -> pt.Response:
        """
        Send a synchronous request to the server.
        """
        request = pt.Request(
            header=pt.RequestHeader(
                task_id=task_id,
                session_id=self.session_id,
                command=command_id,
                version=PTSL_VERSION
            ),
            request_body_json=request_body_json
        )
        response = self.raw_client.SendGrpcRequest(request)
        return response

    def close(self):
        """
        Closes the client.
        """
        self.is_open = False
        self.channel.close()
        self.session_id = ""

    def _primitive_check_if_ready(self) -> bool:
        """
        Checks if the Pro Tools RPC server is listening. The server will
        respond to this command even if the client is not yet authenticated.
        """
        response = self._send_sync_request(pt.HostReadyCheck, "")

        if response.header.status == pt.Failed:
            print("Pro Tools Not Ready")
            print(response)
            return False
        else:
            return True

    def _primitive_register_connection(self, company_name: str,
                                       application_name: str):
        """
        Registers the client's connection to the Pro Tools RPC server.

        This method is called automatically by the initializer.
        """

        req = pt.RegisterConnectionRequestBody(
            company_name=company_name,
            application_name=application_name)

        req_json = json_format.MessageToJson(req,
                                             preserving_proto_field_name=True)

        response = self._send_sync_request(pt.RegisterConnection,
                                           req_json)

        if response.header.status == pt.Failed:
            print("An error occurred")
            print(response)
        else:
            registration_response = \
                json_format.Parse(response.response_body_json,
                                  pt.RegisterConnectionResponseBody())

            self.session_id = registration_response.session_id

    def _primitive_authorize_connection(self, api_key_path) -> Optional[str]:
        """
        (Deprecated) Authorizes the client's connection to the Pro Tools RPC
        server.

        This method is called automatically by the initializer.
        """

        print("WARNING: Certificate authorization method is deprecated",
              file=sys.stderr)

        KEY_FILE_ENCODING = 'ascii'

        with io.FileIO(api_key_path) as f:
            api_token = f.readall().decode(encoding=KEY_FILE_ENCODING)

        req = pt.AuthorizeConnectionRequestBody(auth_string=api_token)
        req_json = json_format.MessageToJson(req,
                                             preserving_proto_field_name=True)

        response = self._send_sync_request(pt.AuthorizeConnection,
                                           req_json)

        if response.header.status == pt.Failed:
            print("An error occurred")
            print(response)
        else:
            authorization_response = \
                json_format.Parse(response.response_body_json,
                                  pt.AuthorizeConnectionResponseBody())

            if authorization_response.is_authorized:
                self.session_id = authorization_response.session_id
            else:
                print("Connection did not authorize, message: " +
                      authorization_response.message)
